from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class SecretStoreRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Secret Store Root"
    certificate_protection_key = Attribute('Certificate Protection Key', min_version='21.4')
    dsn = Attribute('DSN', min_version='21.4')
    driver_name = Attribute('Driver Name', min_version='21.4')
    non_authoritative_classes = Attribute('Non-Authoritative Classes', min_version='21.4')
    pkcs8_association = Attribute('PKCS8 Association', min_version='21.4')
    x509certificate_association = Attribute('X509Certificate Association', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

class Permissions(ObjectModel):
    delete: bool = ApiField(alias='Delete')
    discover: bool = ApiField(alias='Discover')
    manage: bool = ApiField(alias='Manage')
    read: bool = ApiField(alias='Read')
    revoke: bool = ApiField(alias='Revoke')

class DeviceConfiguration(ObjectModel):
    verification_uri: str = ApiField(alias='VerificationUri')
    verification_uri_complete_format: str = ApiField(alias='VerificationUriCompleteFormat')

class CertificateConfiguration(ObjectModel):
    attempt_sidextension_before_source_field: bool = ApiField(alias='AttemptSIDExtensionBeforeSourceField')
    source_field: str = ApiField(alias='SourceField')
    authorized_issuer_dns: list[str] = ApiField(alias='AuthorizedIssuerDns')

class Configuration(ObjectModel):
    session_pool_size: int = ApiField(alias='SessionPoolSize')
    session_pool_age: int = ApiField(alias='SessionPoolAge')
    session_pool_expiration_interval: int = ApiField(alias='SessionPoolExpirationInterval')
    session_rights_refresh_interval: int = ApiField(alias='SessionRightsRefreshInterval')
    strict_expiration: bool = ApiField(alias='StrictExpiration')
    statistics_api_tracking: bool = ApiField(alias='StatisticsApiTracking')
    statistics_tracking_includes_time: bool = ApiField(alias='StatisticsTrackingIncludesTime')
    open_api_enabled: bool = ApiField(alias='OpenApiEnabled')
    open_api_ui: str = ApiField(alias='OpenApiUi')
    default_access_token_validity: int = ApiField(alias='DefaultAccessTokenValidity')
    default_grant_validity: int = ApiField(alias='DefaultGrantValidity')
    default_grant_refreshable: bool = ApiField(alias='DefaultGrantRefreshable')
    unused_access_token_expiration: int = ApiField(alias='UnusedAccessTokenExpiration')
    refresh_endpoint_enabled: bool = ApiField(alias='RefreshEndpointEnabled')
    authorize_by_password: bool = ApiField(alias='AuthorizeByPassword')
    authorize_by_certificate: bool = ApiField(alias='AuthorizeByCertificate')
    authorize_by_integrated: bool = ApiField(alias='AuthorizeByIntegrated')
    authorize_by_jwt: bool = ApiField(alias='AuthorizeByJwt')
    authorize_device: bool = ApiField(alias='AuthorizeDevice')
    device_configuration: DeviceConfiguration = ApiField(alias='DeviceConfiguration')
    certificate_configuration: CertificateConfiguration = ApiField(alias='CertificateConfiguration')

class OwnedApplication(ObjectModel):
    application_id: str = ApiField(alias='ApplicationId')
    role: int = ApiField(alias='Role')

class Role(ObjectModel):
    role: int = ApiField(alias='Role')
    grantee: str = ApiField(alias='Grantee')
    application_id: str = ApiField(alias='ApplicationId')
    permitted_applications: list[str] = ApiField(alias='PermittedApplications')
    owned_applications: list[OwnedApplication] = ApiField(alias='OwnedApplications')

class Application(ObjectModel):
    access_validity: str = ApiField(alias='AccessValidity')
    description: str = ApiField(alias='Description')
    grant_count: str = ApiField(alias='GrantCount')
    grant_validity: str = ApiField(alias='GrantValidity')
    id: str = ApiField(alias='ID')
    maximum_scope: str = ApiField(alias='MaximumScope')
    name: str = ApiField(alias='Name')
    renewable: str = ApiField(alias='Renewable')
    scope: str = ApiField(alias='Scope')
    unused_expiration: str = ApiField(alias='UnusedExpiration')
    url: str = ApiField(alias='Url')
    user_count: str = ApiField(alias='UserCount')
    vendor: str = ApiField(alias='Vendor')

class Scope(ObjectModel):
    scope: str = ApiField(alias='Scope')
    restrictionList: list[str] = ApiField(alias='RestrictionList')

class Grant(ObjectModel):
    scope: str = ApiField(alias='Scope')
    access_token: str = ApiField(alias='AccessToken')
    valid_for: int = ApiField(alias='ValidFor')
    access_issued_on: str = ApiField(alias='AccessIssuedOn')
    access_issued_on_unix_time: int = ApiField(alias='AccessIssuedOnUnixTime')
    access_issued_on_iso_8601: str = ApiField(alias='AccessIssuedOnISO8601')
    refresh_token: str = ApiField(alias='RefreshToken')
    expires: str = ApiField(alias='Expires')
    expires_unix_time: int = ApiField(alias='ExpiresUnixTime')
    expires_iso_8601: str = ApiField(alias='ExpiresISO8601')
    grant_issued_on: str = ApiField(alias='GrantIssuedOn')
    grant_issued_on_unix_time: int = ApiField(alias='GrantIssuedOnUnixTime')
    grant_issued_on_iso_8601: str = ApiField(alias='GrantIssuedOnISO8601')
    application: str = ApiField(alias='Application')
    grantee_prefixed_universal: str = ApiField(alias='GranteePrefixedUniversal')
    granted_to_prefixed_universal: str = ApiField(alias='GrantedToPrefixedUniversal')

class Rule(ObjectModel):
    description: str = ApiField(alias='Description')
    maximum_scope: str = ApiField(alias='MaximumScope')
    access_validity: int = ApiField(alias='AccessValidity')
    grant_validity: int = ApiField(alias='GrantValidity')
    renewable: bool = ApiField(alias='Renewable')
    trustee_prefixed_universal: str = ApiField(alias='TrusteePrefixedUniversal')
    application_id: str = ApiField(alias='ApplicationId')

class JwtMapping(ObjectModel):
    issuer_uri: str = ApiField(alias='IssuerUri')
    purpose_field: str = ApiField(alias='PurposeField')
    purpose_match: str = ApiField(alias='PurposeMatch')
    id_field: str = ApiField(alias='IdField')
    id_match: str = ApiField(alias='IdMatch')
    grantee_prefixed_universal: str = ApiField(alias='GranteePrefixedUniversal')
    name: str = ApiField(alias='Name')



from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import identity as ident

class _Teams(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Teams')
        self.AddTeamMembers = self._AddTeamMembers(api_obj=self._api_obj, url=f'{self._url}/AddTeamMembers')
        self.AddTeamOwners = self._AddTeamOwners(api_obj=self._api_obj, url=f'{self._url}/AddTeamOwners')
        self.DemoteTeamOwners = self._DemoteTeamOwners(api_obj=self._api_obj, url=f'{self._url}/DemoteTeamOwners')
        self.RemoveTeamMembers = self._RemoveTeamMembers(api_obj=self._api_obj, url=f'{self._url}/RemoveTeamMembers')
        self.RenameTeam = self._RenameTeam(api_obj=self._api_obj, url=f'{self._url}/RenameTeam')

    def post(
        self, name: str, owners: list[Union[dict, ident.Identity]], assets: list[str] = None, description: str = None,
        members: list[Union[dict, ident.Identity]] = None, products: list[str] = None
    ):
        body = {
            'Assets'     : assets,
            'Description': description,
            'Name'       : name,
            'Owners'     : owners,
            'Members'    : members,
            'Products'   : products
        }

        class Output(WebSdkOutputModel):
            identity: ident.Identity = ApiField(default_factory=list, alias='ID')
            invalid_members: list[ident.InvalidIdentity] = ApiField(default_factory=list, alias='InvalidMembers')
            invalid_owners: list[ident.InvalidIdentity] = ApiField(alias='InvalidOwners', default_factory=list)
            message: str = ApiField(alias='Message')

        return generate_output(output_cls=Output, response=self._post(data=body))

    class _AddTeamMembers(WebSdkEndpoint):
        def put(
            self, members: list[Union[dict, ident.Identity]], team: Union[dict, ident.Identity] = None,
            show_members: bool = None
        ):
            body = {
                'Members'    : members,
                'Team'       : team,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                invalid_members: list[ident.InvalidIdentity] = ApiField(default_factory=list, alias='InvalidMembers')
                members: list[ident.Identity] = ApiField(default_factory=list, alias='Members')
                message: str = ApiField(alias='Message')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _AddTeamOwners(WebSdkEndpoint):
        def put(
            self,
            owners: list[Union[dict, ident.Identity]] = None,
            team: Union[dict, ident.Identity] = None,
            show_members: bool = None
        ):
            body = {
                'Owners'     : owners,
                'Team'       : team,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                members: list[ident.Identity] = ApiField(default_factory=list, alias='Members')
                message: str = ApiField(alias='Message')
                owners: list[ident.Identity] = ApiField(default_factory=list, alias='Owners')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _DemoteTeamOwners(WebSdkEndpoint):
        def put(
            self, owners: list[Union[dict, ident.Identity]] = None, team: Union[dict, ident.Identity] = None,
            show_members: bool = None
        ):
            body = {
                'Owners'     : owners,
                'Team'       : team,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                invalid_members: list[ident.InvalidIdentity] = ApiField(default_factory=list, alias='InvalidMembers')
                members: list[ident.Identity] = ApiField(default_factory=list, alias='Members')
                message: str = ApiField(alias='Message')
                owners: list[ident.Identity] = ApiField(default_factory=list, alias='Owners')

            return generate_output(output_cls=Output, response=self._put(data=body))

    def Prefix(self, prefix='local'):
        return self._Prefix(api_obj=self._api_obj, url=f'{self._url}/{prefix}')

    class _Prefix(WebSdkEndpoint):
        def Universal(self, universal):
            return self._Universal(api_obj=self._api_obj, url=f'{self._url}/{universal}')

        class _Universal(WebSdkEndpoint):
            def delete(self):
                class Output(WebSdkOutputModel):
                    message: str = ApiField(alias='Message')

                return generate_output(output_cls=Output, response=self._delete())

            def get(self):
                class Output(WebSdkOutputModel):
                    assets: list[str] = ApiField(default_factory=list, alias='Assets')
                    description: str = ApiField(alias='Description')
                    identity: ident.Identity = ApiField(alias='ID')
                    members: list[ident.Identity] = ApiField(default_factory=list, alias='Members')
                    message: str = ApiField(alias='Message')
                    owners: list[ident.Identity] = ApiField(default_factory=list, alias='Owners')
                    products: list[str] = ApiField(default_factory=list, alias='Products')

                return generate_output(output_cls=Output, response=self._get())

            def put(
                self, assets: list[str], description: str, name: str, owners: list[Union[dict, ident.Identity]],
                members: list[Union[dict, ident.Identity]], products: list[str]
            ):
                body = {
                    'Assets'     : assets,
                    'Description': description,
                    'Name'       : name,
                    'Owners'     : owners,
                    'Members'    : members,
                    'Products'   : products
                }

                class Output(WebSdkOutputModel):
                    identity: ident.Identity = ApiField(alias='ID')
                    invalid_owners: list[ident.InvalidIdentity] = ApiField(default_factory=list, alias='InvalidOwners')
                    invalid_members: list[ident.InvalidIdentity] = ApiField(
                        default_factory=list,
                        alias='InvalidMembers'
                    )
                    message: str = ApiField(alias='Message')

                return generate_output(output_cls=Output, response=self._put(data=body))

    class _RemoveTeamMembers(WebSdkEndpoint):
        def put(
            self,
            team: Union[dict, ident.Identity],
            members: list[Union[dict, ident.Identity]] = None,
            show_members: bool = None
        ):
            body = {
                'Team'       : team,
                'Members'    : members,
                'ShowMembers': show_members
            }

            class Output(WebSdkOutputModel):
                invalid_members: list[ident.InvalidIdentity] = ApiField(default_factory=list, alias='InvalidMembers')
                members: list[ident.Identity] = ApiField(default_factory=list, alias='Members')
                message: str = ApiField(alias='Message')
                owners: list[ident.Identity] = ApiField(default_factory=list, alias='Owners')

            return generate_output(output_cls=Output, response=self._put(data=body))

    class _RenameTeam(WebSdkEndpoint):
        def put(self, team: Union[dict, ident.Identity], new_team_name: str):
            body = {
                'Team'       : team,
                'NewTeamName': new_team_name
            }

            class Output(WebSdkOutputModel):
                identity: ident.Identity = ApiField(alias='ID')
                message: str = ApiField(alias='Message')

            return generate_output(output_cls=Output, response=self._put(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class ZoneBaseAttributes(metaclass=IterableMeta):
    __config_class__ = "Zone Base"
    address_range = Attribute('Address Range', min_version='21.4')
    window_end = Attribute('Window End', min_version='21.4')
    window_start = Attribute('Window Start', min_version='21.4')
    zone_contact = Attribute('Zone Contact', min_version='21.4')
    zone_description = Attribute('Zone Description', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class LogApplicationAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Application"
    configuration = Attribute('Configuration')
    customization_data = Attribute('Customization Data')
    log_application_id = Attribute('Log Application ID')
    log_application_name = Attribute('Log Application Name')
    log_application_schema_en = Attribute('Log Application Schema EN')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class SAMLProfileAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "SAML Profile"
    allow_unsolicited_saml_response = Attribute('Allow Unsolicited SAML Response', min_version='21.4')
    application_url = Attribute('Application Url', min_version='21.4')
    binding_type = Attribute('Binding Type', min_version='21.4')
    clock_skew = Attribute('Clock Skew', min_version='21.4')
    default_return_url = Attribute('Default Return Url', min_version='21.4')
    disabled = Attribute('Disabled', min_version='21.4')
    idp_entity_id = Attribute('IdP Entity Id', min_version='21.4')
    idp_metadata_filename = Attribute('IdP Metadata Filename', min_version='21.4')
    idp_metadata_import_date = Attribute('IdP Metadata Import Date', min_version='21.4')
    idp_response_signing_certificate_dn = Attribute('IdP Response Signing Certificate DN', min_version='21.4')
    idp_response_signing_certificate_vault_id = Attribute(
        'IdP Response Signing Certificate Vault Id',
        min_version='21.4'
    )
    idp_signin_url = Attribute('IdP SignIn Url', min_version='21.4')
    idp_vendor = Attribute('IdP Vendor', min_version='21.4')
    min_incoming_signing_algorithm = Attribute('Min Incoming Signing Algorithm', min_version='21.4')
    nameid_format = Attribute('NameId Format', min_version='21.4')
    outbound_signing_algorithm = Attribute('Outbound Signing Algorithm', min_version='21.4')
    relay_state_used_as_return_url = Attribute('Relay State Used As Return Url', min_version='21.4')
    saml_assertion_encryption_requirement = Attribute('SAML Assertion Encryption Requirement', min_version='21.4')
    saml_response_signature_requirement = Attribute('SAML Response Signature Requirement', min_version='21.4')
    sp_decryption_certificate_credential = Attribute('SP Decryption Certificate Credential', min_version='21.4')
    sp_entity_id = Attribute('SP Entity Id', min_version='21.4')
    sp_entity_id_source = Attribute('SP Entity Id Source', min_version='21.4')
    sp_signing_certificate_credential = Attribute('SP Signing Certificate Credential', min_version='21.4')
    sign_authn_requests = Attribute('Sign Authn Requests', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.bulk_application_base import BulkApplicationBaseAttributes

class AdaptableBulkAppAttributes(BulkApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Adaptable Bulk App"
    log_debug = Attribute('Log Debug', min_version='21.4')
    option_1 = Attribute('Option 1', min_version='21.4')
    option_2 = Attribute('Option 2', min_version='21.4')
    pk_credential = Attribute('PK Credential', min_version='21.4')
    password_1 = Attribute('Password 1', min_version='21.4')
    powershell_script = Attribute('PowerShell Script', min_version='21.4')
    powershell_script_hash_vault_id = Attribute('PowerShell Script Hash Vault Id', min_version='21.4')
    script_execution_timeout = Attribute('Script Execution Timeout', min_version='21.4')
    script_hash_mismatch_error = Attribute('Script Hash Mismatch Error', min_version='21.4')
    secondary_credential = Attribute('Secondary Credential', min_version='21.4')
    text_field_1 = Attribute('Text Field 1', min_version='21.4')
    text_field_2 = Attribute('Text Field 2', min_version='21.4')
    text_field_3 = Attribute('Text Field 3', min_version='21.4')
    text_field_4 = Attribute('Text Field 4', min_version='21.4')
    text_field_5 = Attribute('Text Field 5', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class PKCS11Attributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "PKCS11"
    file_validation_disabled = Attribute('File Validation Disabled')
    hsm_cblob = Attribute('HSM:CBlob')
    hsm_cka_label_format = Attribute('HSM:CKA LABEL Format')
    hsm_csr_subject_dn = Attribute('HSM:CSR Subject DN')
    hsm_certificate_directory = Attribute('HSM:Certificate Directory')
    hsm_client_tool_path = Attribute('HSM:Client Tool Path')
    hsm_cryptoki_file = Attribute('HSM:Cryptoki File')
    hsm_embed_sans_in_csr = Attribute('HSM:Embed SANs in CSR')
    hsm_ic_cka_label_format = Attribute('HSM:IC CKA LABEL Format')
    hsm_import_certificate = Attribute('HSM:Import Certificate')
    hsm_issued_id = Attribute('HSM:Issued ID')
    hsm_issued_label = Attribute('HSM:Issued Label')
    hsm_issued_usecase = Attribute('HSM:Issued Usecase')
    hsm_kpblob = Attribute('HSM:KPBlob')
    hsm_last_issued_cblob = Attribute('HSM:Last Issued CBlob')
    hsm_last_issued_id = Attribute('HSM:Last Issued ID')
    hsm_last_issued_kpblob = Attribute('HSM:Last Issued KPBlob')
    hsm_last_issued_label = Attribute('HSM:Last Issued Label')
    hsm_last_issued_usecase = Attribute('HSM:Last Issued Usecase')
    hsm_openssl_config_file = Attribute('HSM:Openssl Config File')
    hsm_openssl_path = Attribute('HSM:Openssl Path')
    hsm_openssl_type = Attribute('HSM:Openssl Type')
    hsm_pkcs11attributes = Attribute('HSM:PKCS11Attributes')
    hsm_protection_type = Attribute('HSM:Protection Type')
    hsm_requested_cka_label = Attribute('HSM:Requested CKA LABEL')
    hsm_requested_ecdh = Attribute('HSM:Requested ECDH')
    hsm_requested_usecase = Attribute('HSM:Requested Usecase')
    hsm_reverse_subject_dn = Attribute('HSM:Reverse Subject DN')
    hsm_tmp_issued_cblob = Attribute('HSM:TMP Issued CBlob')
    hsm_tmp_issued_id = Attribute('HSM:TMP Issued ID')
    hsm_tmp_issued_kpblob = Attribute('HSM:TMP Issued KPBlob')
    hsm_token_label = Attribute('HSM:Token Label')
    hsm_token_password = Attribute('HSM:Token Password')
    hsm_utility_timeout = Attribute('HSM:Utility Timeout')
    network_validation_disabled = Attribute('Network Validation Disabled')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.connection_base import ConnectionBaseAttributes
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes

class BulkApplicationBaseAttributes(
    ConnectionBaseAttributes,
    DriverBaseAttributes,
    ScheduleBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Bulk Application Base"
    batch_size = Attribute('Batch Size')
    certificate_thumbprint = Attribute('Certificate Thumbprint')
    device = Attribute('Device')
    exclude_expired_certificates = Attribute('Exclude Expired Certificates')
    exclude_historical_certificates = Attribute('Exclude Historical Certificates')
    exclude_revoked_certificates = Attribute('Exclude Revoked Certificates')
    grouping_id = Attribute('Grouping Id')
    in_error = Attribute('In Error')
    in_progress = Attribute('In Progress')
    include_certificates_without_private_key = Attribute('Include Certificates Without Private Key')
    last_run = Attribute('Last Run')
    last_update = Attribute('Last Update')
    light_run = Attribute('Light Run')
    light_run_new_certificates_threshold = Attribute('Light Run New Certificates Threshold')
    maximum_days_expired = Attribute('Maximum Days Expired')
    policydn = Attribute('PolicyDN')
    status = Attribute('Status')
    stop_requested = Attribute('Stop Requested')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes

class IdentityDriverAttributes(DriverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Identity Driver"
    display_name_attributes = Attribute('Display Name Attributes')
    mapping_table = Attribute('Mapping Table')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes

class JSSDiscoveryAttributes(TopAttributes, ScheduleBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "JSS Discovery"
    cluster = Attribute('Cluster')
    credential = Attribute('Credential')
    default_container = Attribute('Default Container')
    delete_missing_certificates = Attribute('Delete Missing Certificates')
    delete_missing_clusters = Attribute('Delete Missing Clusters')
    description = Attribute('Description')
    discover_inactive_clusters = Attribute('Discover Inactive Clusters')
    ignore_certificates_valid_less_than = Attribute('Ignore Certificates Valid Less Than')
    import_summary = Attribute('Import Summary')
    include_expired_certificates = Attribute('Include Expired Certificates')
    jss_api_base_url = Attribute('JSS API Base URL')
    jss_auth_audience = Attribute('JSS Auth Audience')
    jss_auth_endpoint = Attribute('JSS Auth Endpoint')
    jss_client_id = Attribute('JSS Client ID')
    jss_client_secret = Attribute('JSS Client Secret')
    last_run = Attribute('Last Run')
    missing_certificates_policy_folder = Attribute('Missing Certificates Policy Folder')
    missing_clusters_policy_folder = Attribute('Missing Clusters Policy Folder')
    retire_missing_certificates = Attribute('Retire Missing Certificates')
    revoke_missing_certificates = Attribute('Revoke Missing Certificates')
    revoke_prevention_period = Attribute('Revoke Prevention Period')
    status = Attribute('Status')
    timeout = Attribute('Timeout')



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

class AdditionalProp(ObjectModel):
    mesh_dns_name: str = ApiField(alias='MeshDnsName')
    engine_identity: str = ApiField(alias='EngineIdentity')
    unreachable_bus_partners: list[str] = ApiField(alias='UnreachableBusPartners')
    current_latency: int = ApiField(alias='CurrentLatency')
    current_workload: int = ApiField(alias='CurrentWorkload')
    standby_mode: int = ApiField(alias='StandbyMode')
    loaded_modules: list[str] = ApiField(alias='LoadedModules')
    process_type: str = ApiField(alias='ProcessType')
    process_id: int = ApiField(alias='ProcessId')
    process_identity: str = ApiField(alias='ProcessIdentity')
    start_time_iso_8601: str = ApiField(alias='StartTimeISO8601')
    last_seen_iso_8601: str = ApiField(alias='LastSeenISO8601')
    sending_host: str = ApiField(alias='SendingHost')
    sending_application: str = ApiField(alias='SendingApplication')
    no_response: bool = ApiField(alias='NoResponse')

class AdditionalPropStatus(ObjectModel):
    additional_prop_1: AdditionalPropStatus = ApiField(alias='additionalProp1')
    additional_prop_2: AdditionalPropStatus = ApiField(alias='additionalProp2')
    additional_prop_3: AdditionalPropStatus = ApiField(alias='additionalProp3')

class Status(ObjectModel):
    reporting_engine: str = ApiField(alias='ReportingEngine')
    mesh: bool = ApiField(alias='Mesh')
    mesh_port: int = ApiField(alias='MeshPort')
    status: str = ApiField(alias='Status')
    status_last_updated_iso_8601: datetime = ApiField(alias='StatusLastUpdatedISO8601')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)

class _Crypto:
    def __init__(self, api_obj):
        self.AvailableKeys = self._AvailableKeys(api_obj=api_obj, url='/Crypto/AvailableKeys')
        self.DefaultKey = self._DefaultKey(api_obj=api_obj, url='/Crypto/DefaultKey')

    class _AvailableKeys(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                keynames: list[str] = ApiField(alias='Keynames', default_factory=list)

            return generate_output(response=self._get(), output_cls=Output)

    class _DefaultKey(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                default_key: str = ApiField(alias='DefaultKey')

            return generate_output(response=self._get(), output_cls=Output)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class GSKAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "GSK"
    backup_store = Attribute('Backup Store', min_version='21.4')
    certificate_label = Attribute('Certificate Label', min_version='21.4')
    create_store = Attribute('Create Store', min_version='21.4')
    default_cert = Attribute('Default Cert', min_version='21.4')
    disable_ssh_history = Attribute('Disable SSH History', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    fips_key = Attribute('Fips Key', min_version='21.4')
    hide_command_line_passwords = Attribute('Hide Command Line Passwords', min_version='21.4')
    java_home_path = Attribute('Java Home Path', min_version='21.4')
    key_store = Attribute('Key Store', min_version='21.4')
    key_store_credential = Attribute('Key Store Credential', min_version='21.4')
    key_store_validation_disabled = Attribute('Key Store Validation Disabled', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    password_expire_days = Attribute('Password Expire Days', min_version='21.4')
    recycle_alias = Attribute('Recycle Alias', min_version='21.4')
    refresh_security = Attribute('Refresh Security', min_version='21.4')
    replace_store = Attribute('Replace Store', min_version='21.4')
    stash_password = Attribute('Stash Password', min_version='21.4')
    store_type = Attribute('Store Type', min_version='21.4')
    temp_certificate_label = Attribute('Temp Certificate Label', min_version='21.4')
    utility_path = Attribute('Utility Path', min_version='21.4')
    version = Attribute('Version', min_version='21.4')



from __future__ import annotations

from typing import (
    Literal,
)

from pyvenafi.tpp.api.websdk.models.codesign import (
    GlobalConfiguration,
    Items,
)
from pyvenafi.tpp.features.bases.feature_base import (
    feature,
    FeatureBase,
)

@feature('Global Configuration')
class CodeSignGlobalConfiguration(FeatureBase):
    def get(
        self
    ) -> GlobalConfiguration:
        """
        Returns settings that apply to all CodeSign Protect projects.

        Returns:
            :class:`~.models.codesign.GlobalConfiguration`
        """
        output = self._api.websdk.Codesign.GetGlobalConfiguration.get()
        return output.global_configuration

    def set(
        self,
        global_configuration: GlobalConfiguration
    ):
        """
        Update global configuration settings for CodeSign Protect.

        Args:
            global_configuration: :class:`~.models.codesign.GlobalConfiguration`
        """
        self._api.websdk.Codesign.SetGlobalConfiguration.post(global_configuration=global_configuration)

    def set_values(
        self,
        approved_key_storage_locations: list[Literal['Software', 'HSM']] = None,
        available_key_storage_locations: list[Literal['Software', 'HSM']] = None,
        default_ca_container: str = None,
        default_certificate_container: str = None,
        default_credential_container: str = None,
        key_use_timeout: int = None,
        project_description_tooltip: str = None,
        request_in_progress_message: str = None,
    ):
        """
        Update specific value(s) on the existing global configuration settings for CodeSign Protect.

        Args:
            approved_key_storage_locations: Key storage locations that are approved for use with CodeSign Protect. This information is also available in the Encryption tree.
            available_key_storage_locations: Available key storage locations, such as HSM and Venafi key storage locations.
            default_ca_container: :ref:`dn` of container where CodeSign Protect creates and stores CA templates.
            default_certificate_container: The default :ref:`dn` of container that holds certificate objects.
            default_credential_container: The default :ref:`dn` of container that holds credentials.
            key_use_timeout: The number of seconds a client should wait for a response when attempting to use a key. The default is 120.
            project_description_tooltip: A description that appears when a project owner is creating a project.
            request_in_progress_message: The message that appears when a KeyUser attempts to access a key that requires approval.
        """

        global_configuration = self.get()
        if any(
            [
                approved_key_storage_locations,
                available_key_storage_locations,
                default_ca_container,
                default_certificate_container,
                default_credential_container,
                key_use_timeout,
                project_description_tooltip,
                request_in_progress_message,
            ]
        ):
            if approved_key_storage_locations:
                global_configuration.approved_key_storage_locations = Items(items=approved_key_storage_locations)
            if available_key_storage_locations:
                global_configuration.available_key_storage_locations = Items(items=available_key_storage_locations)
            if default_ca_container:
                global_configuration.default_ca_container = default_ca_container
            if default_certificate_container:
                global_configuration.default_certificate_container = default_certificate_container
            if default_credential_container:
                global_configuration.default_credential_container = default_credential_container
            if key_use_timeout is not None:
                global_configuration.key_use_timeout = key_use_timeout
            if project_description_tooltip:
                global_configuration.project_description_tooltip = project_description_tooltip
            if request_in_progress_message:
                global_configuration.request_in_progress_message = request_in_progress_message

            self._api.websdk.Codesign.SetGlobalConfiguration.post(global_configuration=global_configuration)



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.agent_discovery_base import AgentDiscoveryBaseAttributes

class AgentCertificateDiscoveryAttributes(AgentDiscoveryBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Agent Certificate Discovery"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class CodeSigningApplicationCollectionAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Application Collection"
    code_signing_application_dn = Attribute('Code Signing Application DN')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class A10AXTrafficManagerAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "A10 AX Traffic Manager"
    bundle_certificate = Attribute('Bundle Certificate')
    certificate_chain_name = Attribute('Certificate Chain Name')
    certificate_name = Attribute('Certificate Name')
    file_validation_disabled = Attribute('File Validation Disabled')
    install_chain_file = Attribute('Install Chain File')
    network_validation_disabled = Attribute('Network Validation Disabled')
    ssh_port = Attribute('SSH Port')
    ssl_cache_size = Attribute('SSL Cache Size')
    ssl_close_notification = Attribute('SSL Close Notification')
    ssl_false_start = Attribute('SSL False Start')
    ssl_pass_phrase = Attribute('SSL Pass Phrase')
    ssl_profile_name = Attribute('SSL Profile Name')
    ssl_profile_type = Attribute('SSL Profile Type')
    tls_version = Attribute('TLS Version')
    use_ssl_template = Attribute('Use SSL Template')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class ClientRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Client Root"
    client_enrollment_anonymous_allowed = Attribute('Client Enrollment Anonymous Allowed', min_version='21.4')
    client_enrollment_default_owner = Attribute('Client Enrollment Default Owner', min_version='21.4')
    client_enrollment_secret_allowed = Attribute('Client Enrollment Secret Allowed', min_version='21.4')
    client_enrollment_secret_dn = Attribute('Client Enrollment Secret DN', min_version='21.4')
    client_portal_signing_certificate = Attribute('Client Portal Signing Certificate', min_version='21.4')
    client_trust_certificate_dn = Attribute('Client Trust Certificate DN', min_version='21.4')
    client_trust_update = Attribute('Client Trust Update', min_version='21.4')
    dsn = Attribute('DSN', min_version='21.4')
    driver_name = Attribute('Driver Name', min_version='21.4')
    environment_variables = Attribute('Environment Variables', min_version='21.4')
    identity_variables = Attribute('Identity Variables', min_version='21.4')
    mac_ignore_list = Attribute('MAC Ignore List', min_version='21.4')
    options = Attribute('Options', min_version='21.4')
    rolling_code_tolerance = Attribute('Rolling Code Tolerance', min_version='21.4')
    rule = Attribute('Rule', min_version='21.4')
    trust_level = Attribute('Trust Level', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.organization import OrganizationAttributes

class PolicyAttributes(OrganizationAttributes, metaclass=IterableMeta):
    __config_class__ = "Policy"
    certificate_origin = Attribute('Certificate Origin', min_version='21.4')
    log_view_server = Attribute('Log View Server', min_version='21.4')
    master_preferences = Attribute('Master Preferences', min_version='21.4')
    scep_ca_ident = Attribute('Scep CA Ident', min_version='21.4')
    scep_certificate_authority = Attribute('Scep Certificate Authority', min_version='21.4')
    scep_challenge_password = Attribute('Scep Challenge Password', min_version='21.4')
    scep_encryption_ra_certificate = Attribute('Scep Encryption RA Certificate', min_version='21.4')
    scep_intune_application_id = Attribute('Scep Intune Application Id', min_version='21.4')
    scep_intune_application_secret = Attribute('Scep Intune Application Secret', min_version='21.4')
    scep_intune_tenant_name = Attribute('Scep Intune Tenant Name', min_version='21.4')
    scep_ra_certificate = Attribute('Scep RA Certificate', min_version='21.4')
    scep_selection_rule = Attribute('Scep Selection Rule', min_version='21.4')
    scep_signing_ra_certificate = Attribute('Scep Signing RA Certificate', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class ClientPortalBaseAttributes(metaclass=IterableMeta):
    __config_class__ = "Client Portal Base"
    client_field_mapping = Attribute('Client Field Mapping', min_version='21.4')
    client_portal_access_enabled = Attribute('Client Portal Access Enabled', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import (
    certificate as cert,
    secret_store,
)

class _X509CertificateStore(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/X509CertificateStore')
        self.Add = self._Add(api_obj=self._api_obj, url=f'{self._url}/Add')
        self.Lookup = self._Lookup(api_obj=self._api_obj, url=f'{self._url}/Lookup')
        self.LookupExpiring = self._LookupExpiring(api_obj=self._api_obj, url=f'{self._url}/LookupExpiring')
        self.Remove = self._Remove(api_obj=self._api_obj, url=f'{self._url}/Remove')
        self.Retrieve = self._Retrieve(api_obj=self._api_obj, url=f'{self._url}/Retrieve')

    class _Add(WebSdkEndpoint):
        def post(
            self, owner_dn: str, certificate_collection_strings: list[str] = None, certificate_string: str = None,
            protection_key: str = None, typed_name_values: list[cert.NameTypeValue] = None
        ):
            body = {
                'CertificateCollectionStrings': certificate_collection_strings,
                'CertificateString'           : certificate_string,
                'OwnerDN'                     : owner_dn,
                'ProtectionKey'               : protection_key,
                'TypedNameValues'             : typed_name_values
            }

            class Output(WebSdkOutputModel):
                leaf_existed: bool = ApiField(alias='LeafExisted')
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_id: int = ApiField(alias='VaultId')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Lookup(WebSdkEndpoint):
        def post(self, certificate_string: str = None, name: str = None, owner_dn: str = None, value: str = None):
            body = {
                'CertificateString': certificate_string,
                'Name'             : name,
                'OwnerDN'          : owner_dn,
                'Value'            : value
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))
                vault_id: int = ApiField(alias='VaultId')
                vault_ids: list[int] = ApiField(alias='VaultIds', default_factory=list)
                certificate_collection_strings: list[str] = ApiField(
                    alias='CertificateCollectionStrings',
                    default_factory=list
                )

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LookupExpiring(WebSdkEndpoint):
        def post(self, days_to_expiration: int, owner_dn: str):
            body = {
                'DaysToExpiration': days_to_expiration,
                'OwnerDN'         : owner_dn
            }

            class Output(WebSdkOutputModel):
                vault_ids: list[int] = ApiField(alias='VaultIds', default_factory=list)
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Remove(WebSdkEndpoint):
        def post(self, owner_dn: str, certificate: str = None, vault_id: int = None):
            body = {
                'Certificate': certificate,
                'OwnerDN'    : owner_dn,
                'VaultId'    : vault_id
            }

            class Output(WebSdkOutputModel):
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Retrieve(WebSdkEndpoint):
        def post(self, vault_id: int):
            body = {
                'VaultId': vault_id
            }

            class Output(WebSdkOutputModel):
                certificate_string: str = ApiField(alias='CertificateString')
                typed_name_values: list[secret_store.TypedNameValues] = ApiField(
                    alias='TypedNameValues',
                    default_factory=list
                )
                result: secret_store.Result = ApiField(alias='Result', converter=lambda x: secret_store.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class HydrantIdv2CAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "HydrantIdv2 CA"
    api_credentials = Attribute('API Credentials', min_version='22.3')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.connection_base import ConnectionBaseAttributes
from pyvenafi.tpp.attributes.ssh_device_base import SshDeviceBaseAttributes
from pyvenafi.tpp.attributes.top import TopAttributes
from pyvenafi.tpp.attributes.validation_base import ValidationBaseAttributes

class DeviceAttributes(
    ConnectionBaseAttributes,
    SshDeviceBaseAttributes,
    TopAttributes,
    ValidationBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Device"
    adaptable_script_max_file_size = Attribute('Adaptable Script Max File Size')
    agentless_discovery_stage = Attribute('Agentless Discovery Stage')
    agentless_discovery_status = Attribute('Agentless Discovery Status')
    allow_adaptable_discovery = Attribute('Allow Adaptable Discovery')
    allow_agentless_discovery_and_remediation = Attribute('Allow Agentless Discovery and Remediation')
    approver = Attribute('Approver')
    bulk_provisioning_dn = Attribute('Bulk Provisioning Dn')
    bulk_provisioning_stage = Attribute('Bulk Provisioning Stage')
    bulk_provisioning_status = Attribute('Bulk Provisioning Status')
    client_id = Attribute('Client ID')
    client_machine_id = Attribute('Client Machine ID')
    cloud_instance_id = Attribute('Cloud Instance ID')
    cloud_region = Attribute('Cloud Region')
    cloud_service = Attribute('Cloud Service')
    created_by = Attribute('Created By')
    deny_multiple_authentication_failures = Attribute('Deny Multiple Authentication Failures')
    disabled_on = Attribute('Disabled On')
    discovered_by_dn = Attribute('Discovered By DN')
    jump_server_dn = Attribute('Jump Server DN')
    last_attempt_to_get_client_subsystem_record = Attribute('Last Attempt To Get Client Subsystem Record')
    last_discovery_date = Attribute('Last Discovery Date')
    last_discovery_planned = Attribute('Last Discovery Planned')
    last_discovery_platform_version = Attribute('Last Discovery Platform Version')
    last_file_operations_platform_version = Attribute('Last File Operations Platform Version')
    location = Attribute('Location')
    log_debug = Attribute('Log Debug')
    manual_approval = Attribute('Manual Approval')
    maximum_authorizations_per_keyset = Attribute('Maximum Authorizations Per Keyset')
    oauth_token_application_id = Attribute('OAuth Token Application Id')
    oauth_token_credential = Attribute('OAuth Token Credential')
    oauth_token_scope = Attribute('OAuth Token Scope')
    onboard_discovery_dn = Attribute('Onboard Discovery Dn')
    onboard_discovery_stage = Attribute('Onboard Discovery Stage')
    onboard_discovery_status = Attribute('Onboard Discovery Status')
    onboard_discovery_to_do = Attribute('Onboard Discovery To Do')
    pbes2_algorithm = Attribute('PBES2 Algorithm')
    placement_id = Attribute('Placement Id')
    placement_job_dn = Attribute('Placement Job Dn')
    powershell_script = Attribute('PowerShell Script')
    powershell_script_hash_vault_id = Attribute('PowerShell Script Hash Vault Id')
    previous_connection_credential_hash = Attribute('Previous Connection Credential Hash')
    privilege_elevation_command = Attribute('Privilege Elevation Command')
    progress = Attribute('Progress')
    protection_key = Attribute('Protection Key')
    required_sync_confirmation = Attribute('Required Sync Confirmation')
    restricted_pbes2_algorithms = Attribute('Restricted PBES2 Algorithms')
    ssh_key_encryption = Attribute('SSH Key Encryption')
    script_execution_timeout = Attribute('Script Execution Timeout')
    status = Attribute('Status')
    system_owned_keys = Attribute('System Owned Keys')
    text_field_1 = Attribute('Text Field 1')
    text_field_10 = Attribute('Text Field 10')
    text_field_11 = Attribute('Text Field 11')
    text_field_12 = Attribute('Text Field 12')
    text_field_2 = Attribute('Text Field 2')
    text_field_3 = Attribute('Text Field 3')
    text_field_4 = Attribute('Text Field 4')
    text_field_5 = Attribute('Text Field 5')
    text_field_6 = Attribute('Text Field 6')
    text_field_7 = Attribute('Text Field 7')
    text_field_8 = Attribute('Text Field 8')
    text_field_9 = Attribute('Text Field 9')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class PersonAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Person"
    full_name = Attribute('Full Name', min_version='21.4')
    given_name = Attribute('Given Name', min_version='21.4')
    group_membership = Attribute('Group Membership', min_version='21.4')
    internet_email_address = Attribute('Internet EMail Address', min_version='21.4')
    language = Attribute('Language', min_version='21.4')
    surname = Attribute('Surname', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)

class SshDeviceBaseAttributes(metaclass=IterableMeta):
    __config_class__ = "Ssh Device Base"
    algorithm = Attribute('Algorithm', min_version='21.4')
    allow_duplicate_private_keys = Attribute('Allow Duplicate Private Keys', min_version='21.4')
    allow_from = Attribute('Allow From', min_version='21.4')
    allow_multiple_authorized_keys = Attribute('Allow Multiple Authorized Keys', min_version='21.4')
    allow_root_access = Attribute('Allow Root Access', min_version='21.4')
    allow_shared_server_accounts = Attribute('Allow Shared Server Accounts', min_version='21.4')
    allow_ssh1 = Attribute('Allow Ssh1', min_version='21.4')
    allow_unencrypted_private_keys = Attribute('Allow Unencrypted Private Keys', min_version='21.4')
    allowed_algorithm = Attribute('Allowed Algorithm', min_version='21.4')
    allowed_command = Attribute('Allowed Command', min_version='21.4')
    allowed_vendor_types = Attribute('Allowed Vendor Types', min_version='21.4')
    automatic_rotation_cleanup_wait = Attribute('Automatic Rotation Cleanup Wait', min_version='21.4')
    automatic_rotation_enabled = Attribute('Automatic Rotation Enabled', min_version='21.4')
    automatic_rotation_interval = Attribute('Automatic Rotation Interval', min_version='21.4')
    automatic_rotation_lead_time = Attribute('Automatic Rotation Lead Time', min_version='21.4')
    deny_from = Attribute('Deny From', min_version='21.4')
    environment = Attribute('Environment', min_version='21.4')
    host_trusts = Attribute('Host Trusts', min_version='21.4')
    key_bit_strength = Attribute('Key Bit Strength', min_version='21.4')
    known_hosts = Attribute('Known Hosts', min_version='21.4')
    management_type = Attribute('Management Type', min_version='21.4')
    maximum_key_age = Attribute('Maximum Key Age', min_version='21.4')
    minimum_key_bit_strength = Attribute('Minimum Key Bit Strength', min_version='21.4')
    required_options = Attribute('Required Options', min_version='21.4')
    root_server_access = Attribute('Root Server Access', min_version='21.4')
    server_access = Attribute('Server Access', min_version='21.4')
    ssh_device_status = Attribute('Ssh Device Status', min_version='21.4')
    ssh_device_type = Attribute('Ssh Device Type', min_version='21.4')
    trusted_root_users = Attribute('Trusted Root Users', min_version='21.4')
    trusted_users = Attribute('Trusted Users', min_version='21.4')
    update_cache = Attribute('Update Cache', min_version='21.4')



from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp import Authenticate
from pyvenafi.tpp.api.websdk.models.codesign import (
    AppleEnvironment,
    CertificateEnvironment,
    CSPEnvironment,
    DotNetEnvironment,
    GPGEnvironment,
    KeyPairEnvironment,
    Rights,
)
from pyvenafi.tpp.api.websdk.models.config import Object
from pyvenafi.tpp.api.websdk.models.identity import Identity
from pyvenafi.tpp.features.bases.feature_base import (
    feature,
    FeatureBase,
)

EnvironmentTypes = Union[
    AppleEnvironment,
    CertificateEnvironment,
    CSPEnvironment,
    DotNetEnvironment,
    GPGEnvironment,
    KeyPairEnvironment,
]

@feature('Rights')
class CodeSignRights(FeatureBase):
    def __init__(
        self,
        api: 'Authenticate'
    ):
        super().__init__(api)

    def add_administrator(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Grant Code Signing Administrator rights to allow a trustee to administer any CodeSign Protect project.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.AddAdministrator.post(prefixed_universal)

    def add_application_administrator(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Grant Code Signing Application Administrator rights to a trustee.
        The Code Signing Application Administrator has permissions to
        :class:`~.models.codesign.Application` and :class:`~.models.codesign.ApplicationCollection`
        objects in CodeSign Protect.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.AddApplicationAdministrator.post(prefixed_universal)

    def add_project_approver(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Grant Project Approval rights to a trustee to sign projects.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.AddProjectApprover.post(prefixed_universal)

    def remove_administrator(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Revoke Code Signing Administrator rights from a trustee.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.RemoveAdministrator.post(prefixed_universal)

    def remove_application_administrator(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Revoke Code Signing Application Administrator rights from a trustee.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.RemoveApplicationAdministrator.post(prefixed_universal)

    def remove_project_approver(
        self,
        trustee: Union[str, Identity]
    ):
        """
        Revoke Key Use Approver rights from a trustee.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        self._api.websdk.Codesign.RemoveProjectApprover.post(prefixed_universal)

    def get_right(
        self,
        obj: Union[str, Object]
    ) -> Rights:
        """
        Returns your rights for a CodeSign Protect object.

        Args:
            obj: :ref:`dn` or :ref:`config_object`

        Returns:
            :class:`~.models.codesign.Rights`
        """
        dn = self._get_dn(obj)
        output = self._api.websdk.Codesign.GetRight.post(dn=dn)
        return output.rights

    def get_trustee_rights(
        self,
        trustee: Union[str, Identity]
    ) -> list[Rights]:
        """
        Returns all CodeSign Protect code signing rights for a user.

        Args:
            trustee: :ref:`identity_object`, :ref:`config_object`, or prefixed universal of an identity.

        Returns:
            :class:`~.models.codesign.Rights`
        """
        prefixed_universal = self._get_prefixed_universal(trustee)
        output = self._api.websdk.Codesign.GetTrusteeRights.post(trustee=prefixed_universal)
        rights_list = [Rights(value=r.value) for r in output.rights_list]
        return rights_list

    def get_object_rights(
        self,
        obj: Union[str, Object]
    ) -> list[Rights]:
        """
        Returns all identities that have rights to a CodeSign Protect object.

        Args:
            obj: :ref:`dn` or :ref:`config_object`

        Returns:
            :class:`~.models.codesign.Rights`
        """
        dn = self._get_dn(obj)
        output = self._api.websdk.Codesign.GetObjectRights.post(dn=dn)
        rights_list = [Rights(value=r.value) for r in output.rights_list]
        return rights_list



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class DigiCertCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "DigiCert CA"
    api_credentials = Attribute('API Credentials', min_version='21.4')
    api_key = Attribute('API Key', min_version='21.4')
    account_number = Attribute('Account Number', min_version='21.4')
    account_organization = Attribute('Account Organization', min_version='21.4')
    allow_reissue = Attribute('Allow Reissue', min_version='21.4')
    certificate_transparency = Attribute('Certificate Transparency', min_version='21.4')
    division = Attribute('Division', min_version='21.4')
    ev_allowed = Attribute('EV Allowed', min_version='21.4')
    ev_enabled = Attribute('EV Enabled', min_version='21.4')
    manual_approval = Attribute('Manual Approval', min_version='21.4')
    organizational_unit = Attribute('Organizational Unit', min_version='21.4')
    profile_id = Attribute('Profile ID', min_version='21.4')
    renewal_window = Attribute('Renewal Window', min_version='21.4')
    san_enabled = Attribute('SAN Enabled', min_version='21.4')
    uc_allowed = Attribute('UC Allowed', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='22.2')
    wildcard_allowed = Attribute('Wildcard Allowed', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.code_signing_environment_base import CodeSigningEnvironmentBaseAttributes

class CodeSigningCertificateEnvironmentAttributes(CodeSigningEnvironmentBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Certificate Environment"
    ca_specific_attributes = Attribute('CA Specific Attributes', min_version='21.4')
    certificate_authority = Attribute('Certificate Authority', min_version='21.4')
    certificate_dn = Attribute('Certificate DN', min_version='21.4')
    city = Attribute('City', min_version='21.4')
    country = Attribute('Country', min_version='21.4')
    elliptic_curve = Attribute('Elliptic Curve', min_version='21.4')
    key_algorithm = Attribute('Key Algorithm', min_version='21.4')
    key_bit_strength = Attribute('Key Bit Strength', min_version='21.4')
    key_storage_location = Attribute('Key Storage Location', min_version='21.4')
    organization = Attribute('Organization', min_version='21.4')
    organizational_unit = Attribute('Organizational Unit', min_version='21.4')
    parameter_set = Attribute('Parameter Set', min_version='24.1')
    provide_chain = Attribute('Provide Chain', min_version='21.4')
    state = Attribute('State', min_version='21.4')
    synchronize_chain = Attribute('Synchronize Chain', min_version='21.4')
    target_store = Attribute('Target Store', min_version='21.4')
    x509_subject = Attribute('X509 Subject', min_version='21.4')
    x509_subjectaltname_rfc822 = Attribute('X509 SubjectAltName RFC822', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.log_channel import LogChannelAttributes

class LogFilterAttributes(LogChannelAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Filter"
    filter_ids = Attribute('Filter IDs', min_version='21.4')
    filter_severity = Attribute('Filter Severity', min_version='21.4')
    log_channel = Attribute('Log Channel', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes

class ReportLicensingAttributes(ReportBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report:Licensing"
    contract_renewal_date = Attribute('Contract Renewal Date', min_version='23.3')
    long_lived_certs_minimum_validity_period = Attribute('Long Lived Certs Minimum Validity Period', min_version='23.3')
    options = Attribute('Options', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.log_sql_channel import LogSQLChannelAttributes

class LogMSSQLAttributes(LogSQLChannelAttributes, metaclass=IterableMeta):
    __config_class__ = "Log MSSQL"



from pyvenafi.cloud.api.session import Session
from pyvenafi.cloud.api.endpoints.account_service import _account_service
from pyvenafi.cloud.api.endpoints.activitylog_service import _activitylog_service
from pyvenafi.cloud.api.endpoints.caoperations_service import _caoperations_service
from pyvenafi.cloud.api.endpoints.certificate_service import _certificate_service
from pyvenafi.cloud.api.endpoints.connectors_service import _connectors_service
from pyvenafi.cloud.api.endpoints.edgemanagement_service import _edgemanagement_service
from pyvenafi.cloud.api.endpoints.integrations_service import _integrations_service
from pyvenafi.cloud.api.endpoints.outagedetection_service import _outagedetection_service
from pyvenafi.cloud.api.endpoints.provisioning_service import _provisioning_service
from pyvenafi.cloud.api.endpoints.tagging_service import _tagging_service

class CloudApi:
    def __init__(
        self, server: str, api_key: str, proxies: dict = None, verify_ssl: bool = False,
        connection_timeout: float = None, read_timeout: float = None
    ):
        self._api_key = api_key
        self._proxies = proxies
        self._verify_ssl = verify_ssl
        self._connection_timeout = connection_timeout
        self._read_timeout = read_timeout
        self._scheme = 'https'

        # region Authentication
        self._base_url = f'{self._scheme}://{server}'
        self._app_url = self._base_url
        self._session = Session(
            headers={
                'Content-Type': 'application/json',
                'tppl-api-key': api_key
            },
            proxies=self._proxies,
            verify_ssl=verify_ssl,
            connection_timeout=connection_timeout,
            read_timeout=read_timeout
        )
        # endregion Authentication

        # region Endpoints
        self.account_service = _account_service(self)
        self.activitylog_service = _activitylog_service(self)
        self.caoperations_service = _caoperations_service(self)
        self.certificate_service = _certificate_service(self)
        self.connectors_service = _connectors_service(self)
        self.edgemanagement_service = _edgemanagement_service(self)
        self.integrations_service = _integrations_service(self)
        self.outagedetection_service = _outagedetection_service(self)
        self.provisioning_service = _provisioning_service(self)
        self.tagging_service = _tagging_service(self)
        # endregion Endpoints



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.agent_certificate_base import AgentCertificateBaseAttributes
from pyvenafi.tpp.attributes.agent_driver_base import AgentDriverBaseAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes

class AgentCertificateDriverAttributes(
    AgentCertificateBaseAttributes,
    AgentDriverBaseAttributes,
    ScheduleBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Agent Certificate Driver"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_base import CredentialBaseAttributes

class PrivateKeyCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Private Key Credential"
    username = Attribute('Username')



from pyvenafi.tpp.attributes._helper import IterableMeta


class Classes(metaclass=IterableMeta):
    acme_account = "ACME Account"
    acme_authorization = "ACME Authorization"
    acme_challenge = "ACME Challenge"
    acme_order = "ACME Order"
    acme_root = "ACME Root"
    aws_ec2_instance_monitor = "AWS EC2 Instance Monitor"
    activedirectory_identity_driver = "ActiveDirectory Identity Driver"
    adaptable_app = "Adaptable App"
    adaptable_bulk_app = "Adaptable Bulk App"
    adaptable_ca = "Adaptable CA"
    adaptable_credential = "Adaptable Credential"
    adaptable_credential_connector = "Adaptable Credential Connector"
    adaptable_flow_action_base = "Adaptable Flow Action Base"
    adaptable_workflow = "Adaptable Workflow"
    agent = "Agent"
    agent_base = "Agent Base"
    agent_certificate_base = "Agent Certificate Base"
    agent_certificate_discovery = "Agent Certificate Discovery"
    agent_certificate_driver = "Agent Certificate Driver"
    agent_container = "Agent Container"
    agent_discovery = "Agent Discovery"
    agent_discovery_base = "Agent Discovery Base"
    agent_driver = "Agent Driver"
    agent_driver_base = "Agent Driver Base"
    agent_module_handler = "Agent Module Handler"
    agent_module_handler_container = "Agent Module Handler Container"
    agent_root = "Agent Root"
    agent_ssh_base = "Agent SSH Base"
    agent_ssh_discovery = "Agent SSH Discovery"
    agent_ssh_driver = "Agent SSH Driver"
    amazon_app = "Amazon App"
    amazon_ca = "Amazon CA"
    amazon_credential = "Amazon Credential"
    apache = "Apache"
    apache_application_group = "Apache Application Group"
    aperture_configuration = "Aperture Configuration"
    application_base = "Application Base"
    application_group = "Application Group"
    associate_key_expiration_action = "Associate Key Expiration Action"
    auto_layout_manager = "Auto Layout Manager"
    automatic_password_credential = "Automatic Password Credential"
    azure_key_vault = "Azure Key Vault"
    basic = "Basic"
    bluecoat_sslva = "BlueCoat SSLVA"
    bluecoat_sslva_trust_store = "BlueCoat SSLVA Trust Store"
    branch_base = "Branch Base"
    bulk_application_base = "Bulk Application Base"
    bulk_provisioning_manager = "Bulk Provisioning Manager"
    ca_container = "CA Container"
    ca_import = "CA Import"
    ca_import_manager = "CA Import Manager"
    ca_root = "CA Root"
    capi = "CAPI"
    capi_trust_store = "CAPI Trust Store"
    csp_key_container = "CSP Key Container"
    certificate_authority_base = "Certificate Authority Base"
    certificate_credential = "Certificate Credential"
    certificate_manager = "Certificate Manager"
    certificate_pre_enrollment = "Certificate Pre-Enrollment"
    certificate_provisioning_work = "Certificate Provisioning Work"
    certificate_revocation = "Certificate Revocation"
    certificate_trust_bundle = "Certificate Trust Bundle"
    certificate_trust_store_base = "Certificate Trust Store Base"
    client_agent_automatic_upgrade_work = "Client Agent Automatic Upgrade Work"
    client_agent_configuration_work = "Client Agent Configuration Work"
    client_agent_device_placement_work = "Client Agent Device Placement Work"
    client_agent_ssh_discovery_work = "Client Agent SSH Discovery Work"
    client_agent_ssh_key_usage_work = "Client Agent SSH Key Usage Work"
    client_agent_ssh_provisioning_work = "Client Agent SSH Provisioning Work"
    client_certificate_discovery_work = "Client Certificate Discovery Work"
    client_certificate_work = "Client Certificate Work"
    client_group = "Client Group"
    client_group_base = "Client Group Base"
    client_group_container = "Client Group Container"
    client_group_root = "Client Group Root"
    client_portal_base = "Client Portal Base"
    client_root = "Client Root"
    client_subgroup = "Client Subgroup"
    client_user_certificate_work = "Client User Certificate Work"
    client_work_base = "Client Work Base"
    client_work_container = "Client Work Container"
    client_work_root = "Client Work Root"
    clientgroup_monitor = "ClientGroup Monitor"
    cloud_instance_monitor = "Cloud Instance Monitor"
    cloud_instance_monitor_driver_base = "Cloud Instance Monitor Driver Base"
    code_sign_manager = "Code Sign Manager"
    code_sign_preapproval_action = "Code Sign PreApproval Action"
    code_signing_action = "Code Signing Action"
    code_signing_admin_approval_action = "Code Signing Admin Approval Action"
    code_signing_apple_environment = "Code Signing Apple Environment"
    code_signing_apple_environment_template = "Code Signing Apple Environment Template"
    code_signing_application = "Code Signing Application"
    code_signing_application_collection = "Code Signing Application Collection"
    code_signing_application_container = "Code Signing Application Container"
    code_signing_application_root = "Code Signing Application Root"
    code_signing_csp_environment = "Code Signing CSP Environment"
    code_signing_csp_environment_template = "Code Signing CSP Environment Template"
    code_signing_certificate_environment = "Code Signing Certificate Environment"
    code_signing_certificate_environment_template = "Code Signing Certificate Environment Template"
    code_signing_delete_object_action = "Code Signing Delete Object Action"
    code_signing_dotnet_environment = "Code Signing DotNet Environment"
    code_signing_dotnet_environment_template = "Code Signing DotNet Environment Template"
    code_signing_environment_base = "Code Signing Environment Base"
    code_signing_environment_container = "Code Signing Environment Container"
    code_signing_environment_root = "Code Signing Environment Root"
    code_signing_environment_template_base = "Code Signing Environment Template Base"
    code_signing_environment_template_update_action = "Code Signing Environment Template Update Action"
    code_signing_flow_container = "Code Signing Flow Container"
    code_signing_flow_root = "Code Signing Flow Root"
    code_signing_gpg_environment = "Code Signing GPG Environment"
    code_signing_gpg_environment_template = "Code Signing GPG Environment Template"
    code_signing_key_pair_environment = "Code Signing Key Pair Environment"
    code_signing_key_pair_environment_template = "Code Signing Key Pair Environment Template"
    code_signing_key_time_constraint = "Code Signing Key Time Constraint"
    code_signing_object_management_action = "Code Signing Object Management Action"
    code_signing_pre_qualified_action = "Code Signing Pre Qualified Action"
    code_signing_project = "Code Signing Project"
    code_signing_project_container = "Code Signing Project Container"
    code_signing_project_root = "Code Signing Project Root"
    code_signing_root = "Code Signing Root"
    code_signing_time_constraint_root = "Code Signing Time Constraint Root"
    comodo_ca = "Comodo CA"
    comodo_ccm = "Comodo CCM"
    connectdirect = "ConnectDirect"
    connectdirect_trust_store = "ConnectDirect Trust Store"
    connection_base = "Connection Base"
    create_key_action = "Create Key Action"
    credential_base = "Credential Base"
    credential_container = "Credential Container"
    credential_driver_base = "Credential Driver Base"
    credential_root = "Credential Root"
    cyberark_credentials_driver = "CyberArk Credentials Driver"
    cyberark_password_credential = "CyberArk Password Credential"
    cyberark_username_password_credential = "CyberArk Username Password Credential"
    dpapi_encryption_driver = "DPAPI Encryption Driver"
    datapower = "DataPower"
    datapower_trust_store = "DataPower Trust Store"
    device = "Device"
    digicert_ca = "DigiCert CA"
    discovery = "Discovery"
    discovery_container = "Discovery Container"
    discovery_exclusion = "Discovery Exclusion"
    discovery_manager = "Discovery Manager"
    discovery_root = "Discovery Root"
    discovery_statistics = "Discovery Statistics"
    driver_base = "Driver Base"
    encryption_driver = "Encryption Driver"
    encryption_root = "Encryption Root"
    engine_root = "Engine Root"
    engine_upgrade = "Engine Upgrade"
    entrust_pki_gateway = "Entrust PKI Gateway"
    entrustnet_ca = "EntrustNET CA"
    exclusion = "Exclusion"
    f5_authentication_bundle = "F5 Authentication Bundle"
    f5_ltm_advanced = "F5 LTM Advanced"
    f5_ltm_advanced_trust_store = "F5 LTM Advanced Trust Store"
    flow = "Flow"
    flow_action_add_team_member = "Flow Action Add Team Member"
    flow_action_approver_base = "Flow Action Approver Base"
    flow_action_base = "Flow Action Base"
    flow_action_config_read_approvers = "Flow Action Config Read Approvers"
    flow_action_direct_approvers = "Flow Action Direct Approvers"
    flow_action_team_owner_approval = "Flow Action Team Owner Approval"
    flow_event_action = "Flow Event Action"
    flow_group = "Flow Group"
    flow_root = "Flow Root"
    gsk = "GSK"
    gsk_trust_store = "GSK Trust Store"
    generational_credential = "Generational Credential"
    generic_credential = "Generic Credential"
    globalsign_mssl_ca = "GlobalSign MSSL CA"
    google_cloud_app = "Google Cloud App"
    google_cloud_ca = "Google Cloud CA"
    google_credential = "Google Credential"
    group = "Group"
    http_ca_base = "HTTP CA Base"
    hashicorp_vault_pki = "HashiCorp Vault PKI"
    hydrantid_ca = "HydrantId CA"
    hydrantidv2_ca = "HydrantIdv2 CA"
    identity_driver = "Identity Driver"
    identity_root = "Identity Root"
    imperva_mx = "Imperva MX"
    integrated_credential_driver = "Integrated Credential Driver"
    intermediate_and_root_certificates = "Intermediate and Root Certificates"
    jks = "JKS"
    jks_trust_store = "JKS Trust Store"
    jss_cluster = "JSS Cluster"
    jss_container = "JSS Container"
    jss_discovery = "JSS Discovery"
    jss_discovery_manager = "JSS Discovery Manager"
    jss_namespace = "JSS Namespace"
    jump_server = "Jump Server"
    key_base = "Key Base"
    key_manager = "Key Manager"
    key_pair = "Key Pair"
    keynectis_sequoia_ca = "Keynectis Sequoia CA"
    ldap_identity_driver = "LDAP Identity Driver"
    layout_driver_base = "Layout Driver Base"
    layout_driver_container = "Layout Driver Container"
    layout_root = "Layout Root"
    layout_rule_base = "Layout Rule Base"
    layout_rule_container = "Layout Rule Container"
    legacy_key_base = "Legacy Key Base"
    license = "License"
    license_root = "License Root"
    log_adaptable = "Log Adaptable"
    log_application = "Log Application"
    log_application_container = "Log Application Container"
    log_application_customization = "Log Application Customization"
    log_channel = "Log Channel"
    log_channel_container = "Log Channel Container"
    log_file = "Log File"
    log_filter = "Log Filter"
    log_heartbeat = "Log Heartbeat"
    log_mssql = "Log MSSQL"
    log_notification = "Log Notification"
    log_notification_container = "Log Notification Container"
    log_root = "Log Root"
    log_smtp = "Log SMTP"
    log_snmp = "Log SNMP"
    log_sql_channel = "Log SQL Channel"
    log_server = "Log Server"
    log_syslog = "Log Syslog"
    metadata_base = "Metadata Base"
    metadata_category = "Metadata Category"
    metadata_choice = "Metadata Choice"
    metadata_container = "Metadata Container"
    metadata_datetime = "Metadata DateTime"
    metadata_identity = "Metadata Identity"
    metadata_list = "Metadata List"
    metadata_root = "Metadata Root"
    metadata_text = "Metadata Text"
    microsoft_ca = "Microsoft CA"
    microsoft_ca_pool = "Microsoft CA Pool"
    monitoring_base = "Monitoring Base"
    monitoring_module = "Monitoring Module"
    netscaler = "NetScaler"
    network_device_certificate_work = "Network Device Certificate Work"
    null_encryption_driver = "Null Encryption Driver"
    oauth_device_authorization_flow = "OAuth Device Authorization Flow"
    oauth_device_authorization_initialization_action = "OAuth Device Authorization Initialization Action"
    oauth_device_authorization_wait_for_identity_action = "OAuth Device Authorization Wait For Identity Action"
    onboard_discovery = "Onboard Discovery"
    onboard_discovery_manager = "Onboard Discovery Manager"
    openssl_ca = "OpenSSL CA"
    opentrust_pki_ca = "OpenTrust PKI CA"
    organization = "Organization"
    out_of_band_ca = "Out Of Band CA"
    pem = "PEM"
    pem_trust_store = "PEM Trust Store"
    pkcs_12 = "PKCS#12"
    pkcs_12_trust_store = "PKCS#12 Trust Store"
    palo_alto_network_fw = "Palo Alto Network FW"
    palo_alto_network_fw_trust_store = "Palo Alto Network FW Trust Store"
    password_credential = "Password Credential"
    person = "Person"
    pkcs11_encryption_driver = "Pkcs11 Encryption Driver"
    placement_job = "Placement Job"
    placement_job_container = "Placement Job Container"
    policy = "Policy"
    private_key_credential = "Private Key Credential"
    proxy = "Proxy"
    proxy_base = "Proxy Base"
    quovadis_ca = "QuoVadis CA"
    rsa_keon_ca = "RSA Keon CA"
    recycle_bin_owner_record = "Recycle Bin Owner Record"
    recycle_bin_root = "Recycle Bin Root"
    recycle_bin_vault_record = "Recycle Bin Vault Record"
    redhat_ca = "Redhat CA"
    remote_access_application = "Remote Access Application"
    remote_access_application_container = "Remote Access Application Container"
    remote_access_root = "Remote Access Root"
    remote_access_scope = "Remote Access Scope"
    remote_access_scope_container = "Remote Access Scope Container"
    report_base = "Report Base"
    report_container = "Report Container"
    report_filter_base = "Report Filter Base"
    report_root = "Report Root"
    report_template = "Report Template"
    report_analytics = "Report:Analytics"
    report_certificate_trends_report = "Report:Certificate Trends Report"
    report_certificateduplication = "Report:CertificateDuplication"
    report_certificateinventory = "Report:CertificateInventory"
    report_certificatenetworkvalidation = "Report:CertificateNetworkValidation"
    report_client_certificate_trends_report = "Report:Client Certificate Trends Report"
    report_clients = "Report:Clients"
    report_custom_report = "Report:Custom Report"
    report_demographics_certificate_authority = "Report:Demographics Certificate Authority"
    report_demographics_key_length = "Report:Demographics Key Length"
    report_demographics_signing_algorithm = "Report:Demographics Signing Algorithm"
    report_demographics_validity_period = "Report:Demographics Validity Period"
    report_demographics_wildcard = "Report:Demographics Wildcard"
    report_enhancedcertificateexpiration = "Report:EnhancedCertificateExpiration"
    report_entitlement = "Report:Entitlement"
    report_expiration = "Report:Expiration"
    report_historicalcertificates = "Report:HistoricalCertificates"
    report_licensing = "Report:Licensing"
    report_ssh_authorized_user = "Report:SSH Authorized User"
    report_ssh_discrepancy = "Report:SSH Discrepancy"
    report_ssh_key_expiration = "Report:SSH Key Expiration"
    report_ssh_server_summary = "Report:SSH Server Summary"
    report_ssh_trust = "Report:SSH Trust"
    report_ssh_user_key = "Report:SSH User Key"
    report_ssh_trends_report = "Report:Ssh Trends Report"
    report_trust = "Report:Trust"
    report_value = "Report:Value"
    reporter_service_module = "Reporter Service Module"
    riverbed_steelhead = "Riverbed SteelHead"
    saml = "SAML"
    saml_profile = "SAML Profile"
    ssh_automatic_rotation_trigger = "SSH Automatic Rotation Trigger"
    ssh_ca_key_pair = "SSH CA Key Pair"
    ssh_ca_key_pair_container = "SSH CA Key Pair Container"
    ssh_ca_template = "SSH CA Template"
    ssh_ca_template_container = "SSH CA Template Container"
    ssh_certificate_adaptable_flow_action = "SSH Certificate Adaptable Flow Action"
    ssh_certificate_base = "SSH Certificate Base"
    ssh_certificate_create_certificate_action = "SSH Certificate Create Certificate Action"
    ssh_certificate_create_public_key_action = "SSH Certificate Create Public Key Action"
    ssh_certificate_issuance_flow_container = "SSH Certificate Issuance Flow Container"
    ssh_certificate_manager = "SSH Certificate Manager"
    ssh_certificate_start_processing_action = "SSH Certificate Start Processing Action"
    ssh_client_certificate = "SSH Client Certificate"
    ssh_host_certificate = "SSH Host Certificate"
    ssh_key = "SSH Key"
    ssh_manager = "SSH Manager"
    ssh_manual_install_key_device = "SSH Manual Install Key Device"
    ssh_server_key = "SSH Server Key"
    ssh_user_key = "SSH User Key"
    ssh_work = "SSH Work"
    sso = "SSO"
    schedule_base = "Schedule Base"
    secret_key = "Secret Key"
    secret_store_root = "Secret Store Root"
    self_signed_ca = "Self Signed CA"
    server_agent_base_device_placement_work = "Server Agent Base Device Placement Work"
    server_agent_cert_device_placement_work = "Server Agent Cert Device Placement Work"
    server_agent_ssh_device_placement_work = "Server Agent SSH Device Placement Work"
    server_certificate_work = "Server Certificate Work"
    service_module = "Service Module"
    spi_logger = "Spi Logger"
    ssh_device_base = "Ssh Device Base"
    statistics_container = "Statistics Container"
    statistics_counter = "Statistics Counter"
    statistics_root = "Statistics Root"
    symmetric_key = "Symmetric Key"
    tealeaf_pca = "Tealeaf PCA"
    tealeafsunimp = "TealeafSunimp"
    top = "Top"
    tree_root = "Tree Root"
    trust = "Trust"
    trustwave_ca = "Trustwave CA"
    untrusted_client_group = "Untrusted Client Group"
    upgrade = "Upgrade"
    upgrade_task = "Upgrade Task"
    upgrades_root = "Upgrades Root"
    user = "User"
    user_preference = "User Preference"
    username_password_credential = "Username Password Credential"
    validation_base = "Validation Base"
    validation_manager = "Validation Manager"
    venafi_platform = "Venafi Platform"
    workflow = "Workflow"
    workflow_container = "Workflow Container"
    workflow_root = "Workflow Root"
    workflow_ticket = "Workflow Ticket"
    x509_certificate = "X509 Certificate"
    x509_certificate_base = "X509 Certificate Base"
    x509_certificate_validation = "X509 Certificate Validation"
    x509_code_signing_certificate = "X509 Code Signing Certificate"
    x509_device_certificate = "X509 Device Certificate"
    x509_intermediate_root_certificate = "X509 Intermediate Root Certificate"
    x509_intermediate_root_certificate_container = "X509 Intermediate Root Certificate Container"
    x509_root_certificate = "X509 Root Certificate"
    x509_root_certificate_container = "X509 Root Certificate Container"
    x509_server_certificate = "X509 Server Certificate"
    x509_time_stamping_certificate = "X509 Time Stamping Certificate"
    x509_user_certificate = "X509 User Certificate"
    zone_base = "Zone Base"
    iplanet = "iPlanet"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class CiscoCSMAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "CiscoCSM"
    create_chain_trustpoints = Attribute('Create Chain Trustpoints')
    create_new_trustpoint = Attribute('Create New Trustpoint')
    enable_credential = Attribute('Enable Credential')
    generate_new_key = Attribute('Generate New Key')
    key_created = Attribute('Key Created')
    key_pair_name = Attribute('Key Pair Name')
    trustpoint_name = Attribute('Trustpoint Name')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.discovery_statistics import DiscoveryStatisticsAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes
from pyvenafi.tpp.attributes.top import TopAttributes

class DiscoveryAttributes(DiscoveryStatisticsAttributes, ScheduleBaseAttributes, TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Discovery"
    address_parsing_errors = Attribute('Address Parsing Errors')
    address_range = Attribute('Address Range')
    automatically_import = Attribute('Automatically Import')
    certificate_instances_found = Attribute('Certificate Instances Found')
    certificate_location_dn = Attribute('Certificate Location DN')
    certificates_already_known = Attribute('Certificates Already Known')
    certificates_excluded = Attribute('Certificates Excluded')
    completed = Attribute('Completed')
    completed_assignments = Attribute('Completed Assignments')
    configuration = Attribute('Configuration')
    connect_excluded = Attribute('Connect Excluded')
    creation_date = Attribute('Creation Date')
    device_location_dn = Attribute('Device Location DN')
    discovery_exclusion_dn = Attribute('Discovery Exclusion DN')
    in_progress = Attribute('In Progress')
    keys_already_known = Attribute('Keys Already Known')
    keys_excluded = Attribute('Keys Excluded')
    last_update = Attribute('Last Update')
    new_certificate_instances_found = Attribute('New Certificate Instances Found')
    new_certificates_found = Attribute('New Certificates Found')
    new_ssh_servers_found = Attribute('New SSH Servers Found')
    placement_preview = Attribute('Placement Preview')
    placement_rule = Attribute('Placement Rule')
    placement_summary = Attribute('Placement Summary')
    priority = Attribute('Priority')
    protection_key = Attribute('Protection Key')
    report_dn = Attribute('Report DN')
    resolve_host = Attribute('Resolve Host')
    ssh_servers_found = Attribute('SSH Servers Found')
    started = Attribute('Started')
    status = Attribute('Status')
    total_assignments = Attribute('Total Assignments')
    window_days_of_month = Attribute('Window Days of Month')
    window_days_of_week = Attribute('Window Days of Week')
    window_end = Attribute('Window End')
    window_start = Attribute('Window Start')
    work_units = Attribute('Work Units')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class TrustwaveCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Trustwave CA"
    interval = Attribute('Interval', min_version='21.4')
    reseller_id = Attribute('Reseller ID', min_version='21.4')
    retrieval_period = Attribute('Retrieval Period', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class OutOfBandCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Out Of Band CA"
    email_contacts = Attribute('Email Contacts', min_version='22.1')
    internet_email_address = Attribute('Internet EMail Address', min_version='22.1')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.metadata_base import MetadataBaseAttributes

class MetadataIdentityAttributes(MetadataBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Metadata Identity"
    single = Attribute('Single', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.password_credential import PasswordCredentialAttributes

class CyberArkPasswordCredentialAttributes(PasswordCredentialAttributes, metaclass=IterableMeta):
    __config_class__ = "CyberArk Password Credential"
    account_name = Attribute('Account Name', min_version='21.4')
    application_id = Attribute('Application ID', min_version='21.4')
    folder = Attribute('Folder', min_version='21.4')
    safe = Attribute('Safe', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class EntrustNETCAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "EntrustNET CA"
    allow_reissue = Attribute('Allow Reissue', min_version='21.4')
    certificate_block = Attribute('Certificate Block', min_version='21.4')
    certificate_transparency = Attribute('Certificate Transparency', min_version='21.4')
    enhanced_key_usage = Attribute('Enhanced Key Usage', min_version='21.4')
    interval = Attribute('Interval', min_version='21.4')
    organization = Attribute('Organization', min_version='21.4')
    retrieval_period = Attribute('Retrieval Period', min_version='21.4')
    use_default_organization = Attribute('Use Default Organization', min_version='21.4')
    username_credential = Attribute('Username Credential', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import provisioning_service

class _provisioning_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.machineidentities = self._machineidentities(
                api_obj=self._api_obj,
                url=f'{self._url}/machineidentities'
            )
            self.machineidentitysearch = self._machineidentitysearch(
                api_obj=self._api_obj,
                url=f'{self._url}/machineidentitysearch'
            )
            self.machines = self._machines(api_obj=self._api_obj, url=f'{self._url}/machines')
            self.machinesearch = self._machinesearch(api_obj=self._api_obj, url=f'{self._url}/machinesearch')
            self.machinetypes = self._machinetypes(api_obj=self._api_obj, url=f'{self._url}/machinetypes')

        class _machineidentities(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self):
                class Output(CloudApiOutputModel):
                    MachineIdentityResponse: provisioning_service.MachineIdentityResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    rc_mapping={
                        200: 'MachineIdentityResponse'
                    }
                )

            def post(self, MachineIdentityCreationRequest: provisioning_service.MachineIdentityCreationRequest):
                data = {**MachineIdentityCreationRequest.dict()}

                class Output(CloudApiOutputModel):
                    MachineIdentityInformation: provisioning_service.MachineIdentityInformation

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        201: 'MachineIdentityInformation'
                    }
                )

            class _ID(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.workflows = self._workflows(api_obj=self._api_obj, url=f'{self._url}/workflows')

                def delete(self):
                    class Output(CloudApiOutputModel):
                        pass

                    return generate_output(output_cls=Output, response=self._delete(params={}))

                def get(self):
                    class Output(CloudApiOutputModel):
                        MachineIdentityInformation: provisioning_service.MachineIdentityInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'MachineIdentityInformation'
                        }
                    )

                def patch(self, MachineIdentityUpdateRequest: provisioning_service.MachineIdentityUpdateRequest):
                    data = {**MachineIdentityUpdateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        MachineIdentityInformation: provisioning_service.MachineIdentityInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._patch(data=data),
                        rc_mapping={
                            200: 'MachineIdentityInformation'
                        }
                    )

                class _workflows(CloudApiEndpoint):
                    def post(self, MachineIdentityWorkflowRequest: provisioning_service.MachineIdentityWorkflowRequest):
                        data = {**MachineIdentityWorkflowRequest.dict()}

                        class Output(CloudApiOutputModel):
                            MachineIdentityWorkflowInformation: provisioning_service.MachineIdentityWorkflowInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._post(data=data),
                            rc_mapping={
                                201: 'MachineIdentityWorkflowInformation'
                            }
                        )

        class _machineidentitysearch(CloudApiEndpoint):
            def post(self, MachineIdentitySearchRequest: provisioning_service.MachineIdentitySearchRequest):
                data = {**MachineIdentitySearchRequest.dict()}

                class Output(CloudApiOutputModel):
                    MachineIdentityDocumentResponse: provisioning_service.MachineIdentityDocumentResponse

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        200: 'MachineIdentityDocumentResponse'
                    }
                )

        class _machines(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self):
                class Output(CloudApiOutputModel):
                    MachinesResponse: provisioning_service.MachinesResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    rc_mapping={
                        200: 'MachinesResponse'
                    }
                )

            def post(self, MachineCreationRequest: provisioning_service.MachineCreationRequest):
                data = {**MachineCreationRequest.dict()}

                class Output(CloudApiOutputModel):
                    MachineInformation: provisioning_service.MachineInformation

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        201: 'MachineInformation'
                    }
                )

            class _ID(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.batchprovisionings = self._batchprovisionings(
                        api_obj=self._api_obj,
                        url=f'{self._url}/batchprovisionings'
                    )
                    self.discovery = self._discovery(api_obj=self._api_obj, url=f'{self._url}/discovery')
                    self.workflows = self._workflows(api_obj=self._api_obj, url=f'{self._url}/workflows')

                def delete(self):
                    class Output(CloudApiOutputModel):
                        pass

                    return generate_output(output_cls=Output, response=self._delete(params={}))

                def get(self):
                    class Output(CloudApiOutputModel):
                        MachineInformation: provisioning_service.MachineInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'MachineInformation'
                        }
                    )

                def patch(self, MachineUpdateRequest: provisioning_service.MachineUpdateRequest):
                    data = {**MachineUpdateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        MachineInformation: provisioning_service.MachineInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._patch(data=data),
                        rc_mapping={
                            200: 'MachineInformation'
                        }
                    )

                class _batchprovisionings(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)
                        self.abort = self._abort(api_obj=self._api_obj, url=f'{self._url}/abort')

                    class _abort(CloudApiEndpoint):
                        def post(self):
                            class Output(CloudApiOutputModel):
                                pass

                            return generate_output(output_cls=Output, response=self._post(data={}))

                class _discovery(CloudApiEndpoint):
                    def __init__(self, *args, **kwargs):
                        super().__init__(*args, **kwargs)
                        self.abort = self._abort(api_obj=self._api_obj, url=f'{self._url}/abort')

                    def get(self):
                        class Output(CloudApiOutputModel):
                            MachineDiscoveryResultInformation: provisioning_service.MachineDiscoveryResultInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'MachineDiscoveryResultInformation'
                            }
                        )

                    class _abort(CloudApiEndpoint):
                        def post(self):
                            class Output(CloudApiOutputModel):
                                MachineInformation: provisioning_service.MachineInformation

                            return generate_output(
                                output_cls=Output,
                                response=self._post(data={}),
                                rc_mapping={
                                    202: 'MachineInformation'
                                }
                            )

                class _workflows(CloudApiEndpoint):
                    def post(self, MachineWorkflowRequest: provisioning_service.MachineWorkflowRequest):
                        data = {**MachineWorkflowRequest.dict()}

                        class Output(CloudApiOutputModel):
                            MachineInformation: provisioning_service.MachineInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._post(data=data),
                            rc_mapping={
                                201: 'MachineInformation'
                            }
                        )

        class _machinesearch(CloudApiEndpoint):
            def post(self, ownershipTree: bool):
                data = {
                    'ownershipTree': ownershipTree,
                }

                class Output(CloudApiOutputModel):
                    MachineDocumentResponse: provisioning_service.MachineDocumentResponse

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        200: 'MachineDocumentResponse'
                    }
                )

        class _machinetypes(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)

            def ID(self, id: str):
                return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

            def get(self):
                class Output(CloudApiOutputModel):
                    MachineTypeResponse: provisioning_service.MachineTypeResponse

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    rc_mapping={
                        200: 'MachineTypeResponse'
                    }
                )

            class _ID(CloudApiEndpoint):
                def get(self):
                    class Output(CloudApiOutputModel):
                        MachineTypeInformation: provisioning_service.MachineTypeInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'MachineTypeInformation'
                        }
                    )



from __future__ import annotations

from typing import Literal

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

ProductType = Literal[
    'CodeSign Protect',
    'Client Protect',
    'Platform',
    'SSH Protect',
    'TLS Protect',
]

class Preference(ObjectModel):
    id: int = ApiField(alias='Id')
    universal: str = ApiField(alias='Universal')
    product: str = ApiField(alias='Product')
    category: str = ApiField(alias='Category')
    name: str = ApiField(alias='Name')
    value: str = ApiField(alias='Value')
    locked: bool = ApiField(alias='Locked')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.client_agent_ssh_provisioning_work import ClientAgentSSHProvisioningWorkAttributes

class ClientAgentSSHKeyUsageWorkAttributes(ClientAgentSSHProvisioningWorkAttributes, metaclass=IterableMeta):
    __config_class__ = "Client Agent SSH Key Usage Work"
    max_row_count = Attribute('Max Row Count', min_version='21.4')



from __future__ import annotations

from pyvenafi.logger import features_logger

class FeatureException(Exception):
    def log(self):
        features_logger.error(self.__str__(), stacklevel=2)

class InvalidFormat(FeatureException):
    ...

class InvalidResultCode(FeatureException):
    def __init__(self, code: int, code_description: str = 'Unknown'):
        super().__init__(f'Expected a valid result code, but got "{code}": {code_description}.')

class ObjectDoesNotExist(FeatureException):
    ...

class UnexpectedValue(FeatureException):
    ...



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes
from pyvenafi.tpp.attributes.proxy import ProxyAttributes

class VerizonCAAttributes(HTTPCABaseAttributes, ProxyAttributes, metaclass=IterableMeta):
    __config_class__ = "Verizon CA"
    admin_email = Attribute('Admin Email')
    admin_firstname = Attribute('Admin Firstname')
    admin_surname = Attribute('Admin Surname')
    admin_telnumber = Attribute('Admin Telnumber')
    certificate_block = Attribute('Certificate Block')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.agent_driver_base import AgentDriverBaseAttributes
from pyvenafi.tpp.attributes.schedule_base import ScheduleBaseAttributes

class AgentDriverAttributes(AgentDriverBaseAttributes, ScheduleBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Agent Driver"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class IntermediateandRootCertificatesAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Intermediate and Root Certificates"
    cdp_aia_verification_enabled_on_creation = Attribute('CDP AIA Verification Enabled On Creation', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class CiscoACEAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "CiscoACE"
    certificate_file = Attribute('Certificate File')
    class_map = Attribute('Class Map')
    context_name = Attribute('Context Name')
    file_validation_disabled = Attribute('File Validation Disabled')
    import_only = Attribute('Import Only')
    include_root_chain = Attribute('Include Root Chain')
    network_validation_disabled = Attribute('Network Validation Disabled')
    non_exportable = Attribute('Non-Exportable')
    policy_map = Attribute('Policy Map')
    private_key_file = Attribute('Private Key File')
    sftp_credential = Attribute('SFTP Credential')
    sftp_host = Attribute('SFTP Host')
    sftp_path = Attribute('SFTP Path')
    sftp_port = Attribute('SFTP Port')
    ssl_proxy_service = Attribute('SSL Proxy Service')
    temp_certificate_label = Attribute('Temp Certificate Label')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import vcamanagement_service

class _vcamanagement_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.distributedissuers = self._distributedissuers(
                api_obj=self._api_obj,
                url=f'{self._url}/distributedissuers'
            )

        class _distributedissuers(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.configurations = self._configurations(api_obj=self._api_obj, url=f'{self._url}/configurations')
                self.intermediatecertificates = self._intermediatecertificates(
                    api_obj=self._api_obj, url=f'{self._url}/intermediatecertificates'
                )
                self.policies = self._policies(api_obj=self._api_obj, url=f'{self._url}/policies')
                self.subcaproviders = self._subcaproviders(api_obj=self._api_obj, url=f'{self._url}/subcaproviders')

            class _configurations(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.regions = self._regions(api_obj=self._api_obj, url=f'{self._url}/regions')

                def ID(self, id: str):
                    return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                def get(self):
                    class Output(CloudApiOutputModel):
                        ConfigurationResponse: vcamanagement_service.ConfigurationResponse

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'ConfigurationResponse'
                        }
                    )

                def post(self, ConfigurationCreateRequest: vcamanagement_service.ConfigurationCreateRequest):
                    data = {**ConfigurationCreateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        ExtendedConfigurationInformation: vcamanagement_service.ExtendedConfigurationInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        rc_mapping={
                            201: 'ExtendedConfigurationInformation'
                        }
                    )

                class _ID(CloudApiEndpoint):
                    def delete(self):
                        class Output(CloudApiOutputModel):
                            ConfigurationDeleteResponse: vcamanagement_service.ConfigurationDeleteResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._delete(params={}),
                            rc_mapping={
                                200: 'ConfigurationDeleteResponse'
                            }
                        )

                    def get(self):
                        class Output(CloudApiOutputModel):
                            ExtendedConfigurationInformation: vcamanagement_service.ExtendedConfigurationInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'ExtendedConfigurationInformation'
                            }
                        )

                    def patch(self, ConfigurationUpdateRequest: vcamanagement_service.ConfigurationUpdateRequest):
                        data = {**ConfigurationUpdateRequest.dict()}

                        class Output(CloudApiOutputModel):
                            ExtendedConfigurationInformation: vcamanagement_service.ExtendedConfigurationInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._patch(data=data),
                            rc_mapping={
                                200: 'ExtendedConfigurationInformation'
                            }
                        )

                class _regions(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            ConfigurationResponse: vcamanagement_service.ConfigurationResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'ConfigurationResponse'
                            }
                        )

            class _intermediatecertificates(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def ID(self, id: str):
                    return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                def get(self):
                    class Output(CloudApiOutputModel):
                        IntermediateCertificateResponse: vcamanagement_service.IntermediateCertificateResponse

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'IntermediateCertificateResponse'
                        }
                    )

                class _ID(CloudApiEndpoint):
                    def get(self):
                        class Output(CloudApiOutputModel):
                            IntermediateCertificateInformation: vcamanagement_service.IntermediateCertificateInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'IntermediateCertificateInformation'
                            }
                        )

            class _policies(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def ID(self, id: str):
                    return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                def get(self):
                    class Output(CloudApiOutputModel):
                        PolicyResponse: vcamanagement_service.PolicyResponse

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'PolicyResponse'
                        }
                    )

                def post(self, PolicyCreateRequest: vcamanagement_service.PolicyCreateRequest):
                    data = {**PolicyCreateRequest.dict()}

                    class Output(CloudApiOutputModel):
                        ExtendedPolicyInformation: vcamanagement_service.ExtendedPolicyInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        rc_mapping={
                            201: 'ExtendedPolicyInformation'
                        }
                    )

                class _ID(CloudApiEndpoint):
                    def delete(self):
                        class Output(CloudApiOutputModel):
                            PolicyDeleteResponse: vcamanagement_service.PolicyDeleteResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._delete(params={}),
                            rc_mapping={
                                200: 'PolicyDeleteResponse'
                            }
                        )

                    def get(self):
                        class Output(CloudApiOutputModel):
                            ExtendedPolicyInformation: vcamanagement_service.ExtendedPolicyInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'ExtendedPolicyInformation'
                            }
                        )

                    def patch(self, PolicyUpdateRequest: vcamanagement_service.PolicyUpdateRequest):
                        data = {**PolicyUpdateRequest.dict()}

                        class Output(CloudApiOutputModel):
                            ExtendedPolicyInformation: vcamanagement_service.ExtendedPolicyInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._patch(data=data),
                            rc_mapping={
                                200: 'ExtendedPolicyInformation'
                            }
                        )

            class _subcaproviders(CloudApiEndpoint):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)

                def ID(self, id: str):
                    return self._ID(api_obj=self._api_obj, url=f'{self._url}/{id}')

                def get(self):
                    class Output(CloudApiOutputModel):
                        SubCaProviderResponse: vcamanagement_service.SubCaProviderResponse

                    return generate_output(
                        output_cls=Output,
                        response=self._get(params={}),
                        rc_mapping={
                            200: 'SubCaProviderResponse'
                        }
                    )

                def post(self, SubCaProviderInformation: vcamanagement_service.SubCaProviderInformation):
                    data = {**SubCaProviderInformation.dict()}

                    class Output(CloudApiOutputModel):
                        SubCaProviderInformation: vcamanagement_service.SubCaProviderInformation

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        rc_mapping={
                            201: 'SubCaProviderInformation'
                        }
                    )

                class _ID(CloudApiEndpoint):
                    def delete(self):
                        class Output(CloudApiOutputModel):
                            SubCaProviderDeleteResponse: vcamanagement_service.SubCaProviderDeleteResponse

                        return generate_output(
                            output_cls=Output,
                            response=self._delete(params={}),
                            rc_mapping={
                                200: 'SubCaProviderDeleteResponse'
                            }
                        )

                    def get(self):
                        class Output(CloudApiOutputModel):
                            SubCaProviderInformation: vcamanagement_service.SubCaProviderInformation

                        return generate_output(
                            output_cls=Output,
                            response=self._get(params={}),
                            rc_mapping={
                                200: 'SubCaProviderInformation'
                            }
                        )

                    def patch(self, SubCaProviderInformation: vcamanagement_service.SubCaProviderInformation):
                        data = {**SubCaProviderInformation.dict()}

                        class Output(CloudApiOutputModel):
                            SubCaProviderUpdateRequest: vcamanagement_service.SubCaProviderUpdateRequest

                        return generate_output(
                            output_cls=Output,
                            response=self._patch(data=data),
                            rc_mapping={
                                200: 'SubCaProviderUpdateRequest'
                            }
                        )



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class RecycleBinRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Recycle Bin Root"
    deletion_daily_max_time = Attribute('Deletion Daily Max Time', min_version='22.2')
    expiration_daily_max_time = Attribute('Expiration Daily Max Time', min_version='22.2')
    expiration_days = Attribute('Expiration Days', min_version='22.2')
    recycle_deletion_query_size = Attribute('Recycle Deletion Query Size', min_version='22.2')
    recycle_deletion_sql_timeout = Attribute('Recycle Deletion SQL Timeout', min_version='22.2')
    recycle_deletion_server_dn = Attribute('Recycle Deletion Server DN', min_version='22.2')
    recycle_deletion_tasks = Attribute('Recycle Deletion Tasks', min_version='22.2')
    recycle_purge_server_dn = Attribute('Recycle Purge Server DN', min_version='22.2')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.key_base import KeyBaseAttributes

class SecretKeyAttributes(KeyBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Secret Key"
    secret_key_renewal_flow = Attribute('Secret Key Renewal Flow', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class F5AuthenticationBundleAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "F5 Authentication Bundle"
    advanced_settings_bundle_name = Attribute('Advanced Settings Bundle Name', min_version='21.4')
    certificates = Attribute('Certificates', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class CodeSigningEnvironmentTemplateBaseAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Environment Template Base"
    environment_template_update_flow_dn = Attribute('Environment Template Update Flow DN', min_version='21.4')
    flow_instance_macro = Attribute('Flow Instance Macro', min_version='21.4')
    key_use_authentication = Attribute('Key Use Authentication', min_version='21.4')
    key_use_flow_dn = Attribute('Key Use Flow DN', min_version='21.4')
    naming_pattern = Attribute('Naming Pattern', min_version='21.4')
    per_user = Attribute('Per User', min_version='21.4')
    require_justification = Attribute('Require Justification', min_version='21.4')
    user_key_import = Attribute('User Key Import', min_version='21.4')
    visible_to = Attribute('Visible To', min_version='21.4')



from __future__ import annotations

from dataclasses import dataclass
from typing import (
    Union,
)

from pyvenafi.tpp.api.websdk.models.codesign import (
    Application,
    ApplicationCollection,
)
from pyvenafi.tpp.api.websdk.models.config import Object
from pyvenafi.tpp.features.bases.feature_base import (
    feature,
    FeatureBase,
)

class _CodeSignApplicationBase(FeatureBase):
    def count_references(
        self,
        application: Union[Application, ApplicationCollection]
    ):
        """
        Get the number of references to a :class:`~.models.codesign.Application`
        or a :class:`~.models.codesign.ApplicationCollection` that are currently in
        use within CodeSign Protect projects.

        Args:
            application: class:`~.models.codesign.Application` or :class:`~.models.codesign.ApplicationCollection`

        Returns:
            Integer
        """
        if isinstance(application, Application):
            key = 'application'
        elif isinstance(application, ApplicationCollection):
            key = 'application_collection'
        else:
            raise ValueError(f'Invalid type {application.__class__.__name__}')
        kwargs = {
            key: application
        }
        output = self._api.websdk.Codesign.CountReferences.post(**kwargs)
        return output.count

    def enumerate_references(
        self,
        application: Application = None,
        application_dn: str = None,
        application_guid: str = None,
        collection: ApplicationCollection = None,
        collection_dn: str = None,
        collection_guid: str = None
    ) -> list[str]:
        """
        Get the :ref:`dn`\s that references a :class:`~.models.codesign.Application`
        or a :class:`~.models.codesign.ApplicationCollection` using one parameter.

        Args:
            application: :class:`~.models.codesign.Application`.
            application_dn: :ref:`dn` of an application.
            application_guid: :ref:`guid` of an application.
            collection: :class:`~.models.codesign.ApplicationCollection`.
            collection_dn: :ref:`dn` of an application collection.
            collection_guid: :ref:`guid` of an application collection.

        Returns:
            list[:ref:`dn`]
        """
        params = [
            application,
            application_dn,
            application_guid,
            collection,
            collection_dn,
            collection_guid,
        ]
        if sum(map(bool, params)) != 1:
            raise ValueError('Please provide only one argument')
        output = self._api.websdk.Codesign.EnumerateReferences.post(
            application=application,
            application_collection=collection,
            application_dn=application_dn,
            application_guid=application_guid,
            collection_dn=collection_dn,
            collection_guid=collection_guid,
        )
        return output.reference_dns

@feature('Application')
class CodeSignApplication(_CodeSignApplicationBase):
    def create(
        self,
        name: str,
        parent_folder: Union[Object, str] = r'\VED\Code Signing\Signing Applications',
        description: str = None,
        hash: str = None,  # noqa
        location: str = None,
        permitted_argument_pattern: str = None,
        signatory_issuer: str = None,
        signatory_subject: str = None,
        size: int = None,
        version: str = None,
        raise_if_already_exists: bool = False,
    ) -> Application:
        """
        Create an application with specified values.

        Args:
            name: Name of the application.
            parent_folder: :ref:`config_object` or :ref:`dn` of the parent folder.
            description: Description of the application.
            hash: The SHA256 checksum of the application binary.
            location: The physical path where the signing application is installed.
            permitted_argument_pattern: The pattern of arguments that the signing application uses to run.
            signatory_issuer: The X.500 certificate issuer that signs the application.
            signatory_subject: The X.500 certificate subject.
            size: The size of the signing application executable.
            version: The signing application version.
            raise_if_already_exists: If the object already exists, raise an error.

        Returns:
            :class:`~.models.codesign.Application`
        """
        parent_dn = self._get_dn(parent_folder)
        dn = rf'{parent_dn}\{name}'
        try:
            self._api.websdk.Codesign.CreateApplication.post(dn=dn)
        except:
            if raise_if_already_exists:
                raise
        application = self.get(dn=dn)

        if any(
            [
                description,
                hash,
                location,
                permitted_argument_pattern,
                signatory_issuer,
                signatory_subject,
                size,
                version,
            ]
        ):
            application.description = description
            application.hash = hash
            application.location = location
            application.permitted_argument_pattern = permitted_argument_pattern
            application.signatory_issuer = signatory_issuer
            application.signatory_subject = signatory_subject
            application.size = size
            application.version = version
            self.update(application=application)
            application = self.get(dn=dn)

        return application

    def update(
        self,
        application: Union[Object, Application]
    ):
        """
        Update an application.

        Args:
            application: The updated :class:`~.models.codesign.Application`.
        """
        self._api.websdk.Codesign.UpdateApplication.post(application=application)

    def rename(
        self,
        application: Union[str, Object, Application],
        new_dn: str
    ):
        """
        Rename an application by giving it a new :ref:`dn`.

        Args:
            application: :class:`~.models.codesign.Application`, :ref:`config_object`, or :ref:`dn` of the application.
            new_dn: The new :ref:`dn` for the application.
        """
        dn = self._get_dn(application)
        self._api.websdk.Codesign.RenameApplication.post(new_dn=new_dn, dn=dn)

    def get(
        self,
        dn: str = None,
        guid: str = None,
        id: str = None
    ) -> Application:
        """
        Get an application using a :ref:`dn`, :ref:`guid`, or Identifier.

        Args:
            dn: :ref:`dn` of the application.
            guid: :ref:`guid` of the application.
            id: Identifier of the application.

        Returns:
            :class:`~.models.codesign.Application`
        """
        if any([dn, guid, id]):
            output = self._api.websdk.Codesign.GetApplication.post(dn=dn, guid=guid, id=id)
        else:
            raise ValueError('You must provide a dn, guid, or id to get an application.')
        if not output.application:
            raise Exception('Application does not exist.')
        return output.application

    def enumerate(
        self,
        _filter: str = None
    ) -> list[Application]:
        """
        Enumerate the applications.

        Args:
            _filter: The partial tool name and optional trailing asterisk (*) wild card.

        Returns:
            list[:class:`~.models.codesign.Application`]
        """
        output = self._api.websdk.Codesign.EnumerateApplications.post(filter=_filter)
        return output.applications

    def delete(
        self,
        application: Union[str, Object, Application]
    ):
        """
        Delete an application.

        Args:
            application: :class:`~.models.codesign.Application`, :ref:`dn` , or :ref:`config_object` of the application.
        """
        dn = self._get_dn(application)
        if dn:
            self._api.websdk.Codesign.DeleteApplication.post(dn=dn)
        else:
            guid = self._get_guid(application)
            if guid:
                self._api.websdk.Codesign.DeleteApplication.post(guid=guid)
            else:
                raise ValueError('Invalid argument to delete the application.')

@feature('Application Collection')
class CodeSignApplicationCollection(FeatureBase):
    def create(
        self,
        name: str,
        parent_folder: Union[Object, str] = r'\VED\Code Signing\Signing Applications',
        applications: list[Union[str, Object, Application]] = None,
        raise_if_already_exists: bool = False,
    ) -> ApplicationCollection:
        """
        Create an application collection.

        Args:
            name: Name of the application collection.
            parent_folder: :ref:`dn` of the parent folder.
            applications: List of applications to include in the application collection.
            raise_if_already_exists: If the object already exists, raise an error.

        Returns:
            :class:`~.models.codesign.ApplicationCollection`
        """
        parent_dn = self._get_dn(parent_folder)
        dn = rf'{parent_dn}\{name}'
        try:
            self._api.websdk.Codesign.CreateApplicationCollection.post(dn=dn)
        except:
            if raise_if_already_exists:
                raise

        collection = self.get(dn=dn)
        if applications:
            application_dns = [self._get_dn(a) for a in applications]
            collection.application_dns.items = application_dns
            self.update(collection=collection)
            collection = self.get(dn=dn)

        return collection

    def update(
        self,
        collection: Union[Object, ApplicationCollection]
    ):
        """
        Update an application collection.

        Args:
            collection: The updated :class:`~.models.codesign.ApplicationCollection`.
        """
        self._api.websdk.Codesign.UpdateApplicationCollection.post(application_collection=collection)

    def rename(
        self,
        collection: Union[str, Object, Application],
        new_dn: str
    ):
        """
        Rename an application collection object by giving it a new :ref:`dn`.

        Args:
            collection: :class:`~.models.codesign.ApplicationCollection`, :ref:`config_object`, or :ref:`dn` of the application collection object.
            new_dn: The new :ref:`dn` for the application collection.
        """
        dn = self._get_dn(collection)
        self._api.websdk.Codesign.RenameApplicationCollection.post(new_dn=new_dn, dn=dn)

    def get(
        self,
        dn: str = None,
        guid: str = None,
        id: str = None
    ) -> ApplicationCollection:
        """
        Get an application collection object using a :ref:`dn`, :ref:`guid`, or Identifier.

        Args:
            dn: :ref:`dn` of the application collection object.
            guid: :ref:`guid` of the application collection object.
            id: Identifier of the application collection object.

        Returns:
            :class:`~.models.codesign.ApplicationCollection`
        """
        if any([dn, guid, id]):
            output = self._api.websdk.Codesign.GetApplicationCollection.post(dn=dn, guid=guid, id=id)
        else:
            raise ValueError('You must provide a dn, guid, or id to get an application collection.')
        if not output.application_collection:
            raise Exception('Application collection does not exist.')
        return output.application_collection

    def get_members(
        self,
        dn: str = None,
        guid: str = None,
        id: str = None
    ) -> ApplicationCollection:
        """
        Get :ref:`dn`\s of applications and application collections that are members of an application collection.

        Args:
            dn: :ref:`dn` of the application collection object.
            guid: :ref:`guid` of the application collection object.
            id: Identifier of the application collection object.

        Returns:
            list[:ref:`dn`\s.]
        """
        if any([dn, guid, id]):
            output = self._api.websdk.Codesign.GetApplicationCollectionMembers.post(dn=dn, guid=guid, id=id)
        else:
            raise ValueError("You must provide a dn, guid, or id to get an application collection's members.")
        if not output.application_collection:
            raise Exception('Application collection does not exist.')
        return output.application_collection.application_dns.items

    def get_member_dns(
        self,
        dn: str = None,
        guid: str = None,
        id: str = None
    ):
        """
        Get :ref:`dn`\s of applications and application collections that are members of an application collection.

        Args:
            dn: :ref:`dn` of the application collection object.
            guid: :ref:`guid` of the application collection object.
            id: Identifier of the application collection object.

        Returns:
            Dataclass object containing:
            * :class:`~.models.codesign.ApplicationCollection`
            * list[:ref:`dn`] of application collections.
            * list[:ref:`dn`] of applications.
        """
        if any([dn, guid, id]):
            output = self._api.websdk.Codesign.GetApplicationCollectionMemberDNs.post(dn=dn, guid=guid, id=id)
        else:
            raise ValueError("You must provide a dn, guid, or id to get an application collection's members.")

        @dataclass
        class ApplicationCollectionDns:
            application_collection: ApplicationCollection = output.application_collection
            application_collection_dns: list[str] = output.application_collection_dns
            application_dns: list[str] = output.application_dns

        return ApplicationCollectionDns(
            application_collection=output.application_collection,
            application_collection_dns=output.application_collection_dns,
            application_dns=output.application_dns
        )

    def enumerate(
        self,
        _filter: str = None
    ) -> list[ApplicationCollection]:
        """
        Enumerate the application collections.

        Args:
            _filter: The partial tool name and optional trailing asterisk (*) wild card.

        Returns:
            list[:class:`~.models.codesign.ApplicationCollection`]
        """
        output = self._api.websdk.Codesign.EnumerateApplicationCollections.post(filter=_filter)
        return output.application_collections

    def delete(
        self,
        collection: Union[str, Object, ApplicationCollection]
    ):
        """
        Delete an application collection.

        Args:
            collection: :class:`~.models.codesign.ApplicationCollection`, :ref:`dn`, or :ref:`config_object` of the application.
        """
        dn = self._get_dn(collection)
        if dn:
            self._api.websdk.Codesign.DeleteApplicationCollection.post(dn=dn)
        else:
            guid = self._get_guid(collection)
            if guid:
                self._api.websdk.Codesign.DeleteApplicationCollection.post(guid=guid)
            else:
                raise ValueError('Invalid argument to delete the application collection.')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.x509_root_certificate import X509RootCertificateAttributes

class X509IntermediateRootCertificateAttributes(X509RootCertificateAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Intermediate Root Certificate"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.service_module import ServiceModuleAttributes

class KmipServiceModuleAttributes(ServiceModuleAttributes, metaclass=IterableMeta):
    __config_class__ = "Kmip Service Module"
    agent_module_handlers = Attribute('Agent Module Handlers')
    agent_upgrade_rule = Attribute('Agent Upgrade Rule')
    credential = Attribute('Credential')
    interval = Attribute('Interval')
    port = Attribute('Port')
    start_time = Attribute('Start Time')
    upgrade = Attribute('Upgrade')
    version = Attribute('Version')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class BlueCoatSSLVAAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "BlueCoat SSLVA"
    certificate_label = Attribute('Certificate Label', min_version='21.4')
    certificate_oid = Attribute('Certificate OID', min_version='21.4')
    certificate_only = Attribute('Certificate Only', min_version='21.4')
    create_lists = Attribute('Create Lists', min_version='21.4')
    device_certificate = Attribute('Device Certificate', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    replace_store = Attribute('Replace Store', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class F5Attributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "F5"
    bundle_certificate = Attribute('Bundle Certificate')
    certificate_chain_name = Attribute('Certificate Chain Name')
    certificate_name = Attribute('Certificate Name')
    file_validation_disabled = Attribute('File Validation Disabled')
    fips_key = Attribute('Fips Key')
    network_validation_disabled = Attribute('Network Validation Disabled')
    overwrite_certificate = Attribute('Overwrite Certificate')
    overwrite_existing_chain = Attribute('Overwrite Existing Chain')
    overwrite_key = Attribute('Overwrite Key')
    private_key_name = Attribute('Private Key Name')
    private_key_type = Attribute('Private Key Type')
    ssh_port = Attribute('SSH Port')



from __future__ import annotations

from typing import (
    Generic,
    Optional,
    TypeVar,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

T = TypeVar('T')

# region Models
# region Outputs
class Result(ObjectModel):
    code: int = ApiField()

    @property
    def config_result(self):
        return ResultCodes.Config.get(self.code, 'Unknown')

class NameValues(ObjectModel, Generic[T]):
    name: str = ApiField(alias='Name')
    values: list[T] = ApiField(alias='Values', default_factory=list)

class Object(ObjectModel):
    absolute_guid: str = ApiField(alias='AbsoluteGUID')
    config_id: Optional[int] = ApiField(alias='Id')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='DN')
    guid: str = ApiField(alias='GUID')
    id: int = ApiField(alias='Id')
    in_error: bool = ApiField(alias='InError')
    name: str = ApiField(alias='Name')
    parent: str = ApiField(alias='Parent')
    properties: int = ApiField(alias='Properties')
    revision: Optional[int] = ApiField(alias='Revision')
    type_name: str = ApiField(alias='TypeName')

    def __str__(self):
        return self.dn

class Policy(ObjectModel):
    attribute_name: str = ApiField(alias='AttributeName')
    guid: str = ApiField(alias='GUID')
    property: str = ApiField(alias='Property')
    type_name: str = ApiField(alias='TypeName')
    value_list: list[str] = ApiField(alias='ValueList', default_factory=list)

class ClassDefinition(ObjectModel):
    property: int = ApiField(alias='Property')
    name: str = ApiField(alias='Name')
    super_class_names: list[str] = ApiField(alias='SuperClassNames')
    containment_names: list[str] = ApiField(alias="ContainmentNames")
    containment_sub_names: list[str] = ApiField(alias="ContainmentSubNames")
    naming_names: list[str] = ApiField(alias="NamingNames")
    mandatory_names: list[str] = ApiField(alias="MandatoryNames")
    optional_names: list[str] = ApiField(alias="OptionalNames")
    optional_names_ex: list[str] = ApiField(alias="OptionalNamesEx")
    mandatories_names_ex: list[str] = ApiField(alias="MandatoriesNamesEx")

class AttributeDefinition(ObjectModel):
    syntax: int = ApiField(alias='Syntax')
    property: int = ApiField(alias='Property')
    name: str = ApiField(alias='Name')

# endregion Outputs


# region Inputs
class NameAttribute(ObjectModel, Generic[T]):
    name: str = ApiField(alias='Name')
    value: T = ApiField(alias='Value')
# endregion Inputs
# endregion Models



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.log_channel import LogChannelAttributes

class LogSplunkAttributes(LogChannelAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Splunk"
    credential = Attribute('Credential')
    host = Attribute('Host')
    index = Attribute('Index')
    port = Attribute('Port')
    source = Attribute('Source')
    timeout = Attribute('Timeout')
    verbose = Attribute('Verbose')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_base import CredentialBaseAttributes

class AutomaticPasswordCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Automatic Password Credential"
    self_destruct = Attribute('Self Destruct')
    usage_count = Attribute('Usage Count')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    CloudApiEndpoint,
    CloudApiOutputModel,
    generate_output,
)
from pyvenafi.cloud.api.models import activitylog_service
from typing import List

class _activitylog_service:
    def __init__(self, api_obj):
        self.v1 = self._v1(api_obj=api_obj)

    class _v1(CloudApiEndpoint):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='v1')
            self.activitylogsearch = self._activitylogsearch(
                api_obj=self._api_obj,
                url=f'{self._url}/activitylogsearch'
            )
            self.activitytypes = self._activitytypes(api_obj=self._api_obj, url=f'{self._url}/activitytypes')

        class _activitylogsearch(CloudApiEndpoint):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.export = self._export(api_obj=self._api_obj, url=f'{self._url}/export')

            def post(self, ActivityLogFilter: activitylog_service.ActivityLogFilter):
                data = {**ActivityLogFilter.dict()}

                class Output(CloudApiOutputModel):
                    ActivityLogEntriesResponse: activitylog_service.ActivityLogEntriesResponse

                return generate_output(
                    output_cls=Output,
                    response=self._post(data=data),
                    rc_mapping={
                        200: 'ActivityLogEntriesResponse'
                    }
                )

            class _export(CloudApiEndpoint):
                def post(self, ActivityLogFilter: activitylog_service.ActivityLogFilter):
                    data = {**ActivityLogFilter.dict()}

                    class Output(CloudApiOutputModel):
                        ExportedActivityLogEntryInformationList: List[
                            activitylog_service.ExportedActivityLogEntryInformation]

                    return generate_output(
                        output_cls=Output,
                        response=self._post(data=data),
                        root_field='ExportedActivityLogEntryInformationList'
                    )

        class _activitytypes(CloudApiEndpoint):
            def get(self):
                class Output(CloudApiOutputModel):
                    ActivityLogTypeList: List[activitylog_service.ActivityLogType]

                return generate_output(
                    output_cls=Output,
                    response=self._get(params={}),
                    root_field='ActivityLogTypeList'
                )



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class SSHCATemplateAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "SSH CA Template"
    allow_private_key_reuse = Attribute('Allow Private Key Reuse')
    allow_to_specify_certificate_identifier_on_request = Attribute('Allow To Specify Certificate Identifier On Request')
    allow_to_specify_extensions_on_request = Attribute('Allow To Specify Extensions On Request')
    allow_to_specify_force_command_on_request = Attribute('Allow To Specify Force Command On Request')
    allow_to_specify_principals_on_request = Attribute('Allow To Specify Principals On Request')
    allow_to_specify_source_addresses_on_request = Attribute('Allow To Specify Source Addresses On Request')
    allowed_custom_extensions = Attribute('Allowed Custom Extensions')
    allowed_extension = Attribute('Allowed Extension')
    allowed_forced_command_pattern = Attribute('Allowed Forced Command Pattern')
    allowed_principal_pattern = Attribute('Allowed Principal Pattern')
    allowed_private_key_algorithm = Attribute('Allowed Private Key Algorithm')
    allowed_source_address = Attribute('Allowed Source Address')
    certificate_container = Attribute('Certificate Container')
    certificate_identifier = Attribute('Certificate Identifier')
    certificate_issuance_timeout_before_returning_pending_response = Attribute(
        'Certificate Issuance Timeout Before Returning Pending Response'
    )
    certificate_object_naming_pattern = Attribute('Certificate Object Naming Pattern')
    certificate_type = Attribute('Certificate Type')
    extension = Attribute('Extension')
    fingerprint_sha256 = Attribute('Fingerprint SHA256')
    force_command = Attribute('Force Command')
    hash_algorithm = Attribute('Hash Algorithm')
    include_issued_certificate_in_request = Attribute('Include Issued Certificate In Request')
    include_requester_identity_to_contacts = Attribute('Include Requester Identity to Contacts')
    key_algorithm = Attribute('Key Algorithm')
    key_id_format = Attribute('Key ID Format')
    maximum_age = Attribute('Maximum Age')
    principal = Attribute('Principal')
    public_signing_key_vault_id = Attribute('Public Signing Key Vault Id')
    ssh_certificate_issuance_flow = Attribute('SSH Certificate Issuance Flow')
    signing_key_algorithm = Attribute('Signing Key Algorithm')
    signing_key_dn = Attribute('Signing Key DN')
    source_address = Attribute('Source Address')
    use_certificate_object_naming_pattern = Attribute('Use Certificate Object Naming Pattern')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes

class ComodoCCMAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Comodo CCM"
    customer_login_uri = Attribute('Customer Login URI', min_version='21.4')
    organization = Attribute('Organization', min_version='21.4')
    secret_key = Attribute('Secret Key', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_group import ApplicationGroupAttributes

class ApacheApplicationGroupAttributes(ApplicationGroupAttributes, metaclass=IterableMeta):
    __config_class__ = "Apache Application Group"
    client_tools_path = Attribute('Client Tools Path', min_version='21.4')
    kmdata_key_file = Attribute('KMDATA Key File', min_version='22.2')
    ocs_identifier = Attribute('OCS Identifier', min_version='21.4')
    partition_password_credential = Attribute('Partition Password Credential', min_version='21.4')
    private_key_label = Attribute('Private Key Label', min_version='21.4')
    private_key_location = Attribute('Private Key Location', min_version='21.4')
    protection_type = Attribute('Protection Type', min_version='21.4')
    softcard_identifier = Attribute('Softcard Identifier', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class IIS5Attributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "IIS5"
    file_validation_disabled = Attribute('File Validation Disabled')
    network_validation_disabled = Attribute('Network Validation Disabled')
    site_id = Attribute('Site Id')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes

class ReportClientsAttributes(ReportBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report:Clients"



from __future__ import annotations

from datetime import datetime

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

class Engine(ObjectModel):
    dn: str = ApiField(alias='Dn')
    display_name: str = ApiField(alias='DisplayName')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    name: str = ApiField(alias='Name')

class IisService(ObjectModel):
    modules: list[str] = ApiField(alias='modules', default_factory=list)
    time_since_first_seen: datetime = ApiField(alias='timeSinceFirstSeen')
    time_since_last_seen: datetime = ApiField(alias='timeSinceLastSeen')
    status: str = ApiField(alias='status')

class LogServerService(IisService):
    ...  # This is currently the same.

class vPlatformService(ObjectModel):
    configured_latency: int = ApiField(alias='configuredLatency')
    configured_mode: str = ApiField(alias='configuredMode')
    configured_work: int = ApiField(alias='configuredWork')
    current_latency: int = ApiField(alias='currentLatency')
    current_mode: str = ApiField(alias='currentMode')
    current_work: int = ApiField(alias='currentWork')
    modules: list[str] = ApiField(alias='modules', default_factory=list)
    status: str = ApiField(alias='status')
    time_since_first_seen: datetime = ApiField(alias='timeSinceFirstSeen')
    time_since_last_seen: datetime = ApiField(alias='timeSinceLastSeen')

class Services(ObjectModel):
    vplatform: vPlatformService = ApiField(alias='Vplatform')
    log_server: LogServerService = ApiField(alias='LogServer')
    iis: IisService = ApiField(alias='Iis')

class SystemStatus(ObjectModel):
    engine_dn: str = ApiField(alias='engineDn')
    engine_name: str = ApiField(alias='engineName')
    services: Services = ApiField(alias='services')
    version: str = ApiField(alias='version')

class Task(ObjectModel):
    display_name: str = ApiField(alias='DisplayName')
    name: str = ApiField(alias='Name')
    start_time: datetime = ApiField(alias='StartTime')
    stop_time: datetime = ApiField(alias='StopTime')
    warning_count: int = ApiField(alias='WarningCount')

class UpgradeInfo(ObjectModel):
    id: str = ApiField(alias='Id')
    start_time: datetime = ApiField(alias='StartTime')
    versions: list[str] = ApiField(alias='Versions', default_factory=list)

class UpgradeStatus(ObjectModel):
    engine: 'Engine' = ApiField(alias='Engine')
    status: str = ApiField(alias='Status')
    upgrade_start_time: datetime = ApiField(alias='UpgradeStartTime')
    upgrade_stop_time: datetime = ApiField(alias='UpgradeStopTime')
    tasks_completed: list[Task] = ApiField(alias='TasksCompleted', default_factory=list)
    tasks_pending: list[Task] = ApiField(alias='TasksPending', default_factory=list)
    tasks_running: list[Task] = ApiField(alias='TasksRunning', default_factory=list)

class UpgradeSummary(ObjectModel):
    status: str = ApiField(alias='Status')
    upgrade_start_time: datetime = ApiField(alias='UpgradeStartTime')
    upgrade_stop_time: datetime = ApiField(alias='UpgradeStopTime')
    completed_tasks: int = ApiField(alias='CompletedTasks')
    target_version: str = ApiField(alias='TargetVersion')
    engines_complete: int = ApiField(alias='EnginesComplete')
    engines_running: int = ApiField(alias='EnginesRunning')
    engines_blocked: int = ApiField(alias='EnginesBlocked')
    engines_in_error: int = ApiField(alias='EnginesInError')
    engines_pending_install: int = ApiField(alias='EnginesPendingInstall')
